#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function
from six import string_types



import libsbml
from math import isinf, isnan
from sympy import srepr
from libsignetsim.model.math.sympy_shortcuts import *
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.ModelException import (SbmlException, TagNotImplementedModelException)
from libsignetsim.model.math.MathException import MathException

class SbmlMathReader(object):
	""" Class for handling math formulaes """


	def __init__(self, model):
		""" Constructor """

		self.model = model


	def readSbml(self, formula, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Import a math formula from sbml """

		self.sbmlLevel = sbml_level
		self.sbmlVersion = sbml_version

		self.internalTree = self.translateForInternal(formula, sbml_level, sbml_version)

		if Settings.verbose >= 2:
			print("\n> readSbml : ")
			print(">> input : %s" % self.printSbml(formula))
			print(">> output simplified : %s" % str(self.internalTree))
			print(">> output : %s" % srepr(self.internalTree))

	def printSbml(self, formula, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if isinstance(formula, string_types):
			return formula
		elif isinstance(formula, libsbml.ASTNode):
			if sbml_level <= 2:
				return libsbml.formulaToString(formula)
			else:
				return libsbml.formulaToL3String(formula)
		else:
			return str(formula)


	def translateVariableForInternal(self, variable, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion, simplified=False, develop=True):

		# Local parameter
		# If we are within a reaction, we first need to look in the local
		# scope for the variable identifier

		if self.isFromReaction is not None and self.model.listOfReactions[self.isFromReaction].listOfLocalParameters.containsSbmlId(variable):
			if simplified:
				return SympySymbol("_parameter_")
			else:
				return self.model.listOfReactions[self.isFromReaction].listOfLocalParameters.getBySbmlId(variable).symbol.getInternalMathFormula()


		# And if we are within a function definition, we will encounter
		# variables which are only local
		elif self.isFunctionDefinition:
			return SympySymbol(variable)

		# Finally, we look in our list of variables
		elif self.model.listOfVariables.containsSbmlId(variable):

			t_variable = self.model.listOfVariables.getBySbmlId(variable)

			if t_variable.isStoichiometry():
				if simplified:
					return SympyInteger(1)
				else:
					return t_variable.symbol.getInternalMathFormula()

			elif t_variable.isCompartment():
				if simplified:
					return SympyInteger(1)
				else:
					return t_variable.symbol.getInternalMathFormula()

			elif t_variable.isParameter():
				if simplified:
					return SympySymbol("_parameter_")
				else:
					return t_variable.symbol.getInternalMathFormula()

			elif t_variable.isReaction():
				if simplified:
					return SympySymbol("_reaction_")
				else:
					return t_variable.symbol.getInternalMathFormula()

			elif t_variable.isSpecies():
				if simplified:
					return SympySymbol("_species_")

				elif not develop:
					return t_variable.symbol.getInternalMathFormula()

				elif (self.isKineticLaw
						or self.isRateRule
						or self.isEquation
						or self.isEventAssignment
						or self.isAssignmentRule
						or self.isAlgebraicRule
					  	or self.isDelay
						or self.isPriority
				):

					# If the species represents a concentration
					# We should make it corresponds to amount/size
					if t_variable.isConcentration():

						return SympySymbol("_speciesForcedConcentration_%s_"
											% str(t_variable.symbol.getInternalMathFormula()))

					# Otherwise we use amount
					else:
						return t_variable.symbol.getInternalMathFormula()

				else:
					return t_variable.symbol.getInternalMathFormula()

			else:
				raise SbmlException("SbmlMathReader : Unknown type of variable : %s" % variable)

		elif variable == "_time_":
			return SympySymbol("_time_")

		else:
			raise SbmlException("SbmlMathReader : Unknown variable : %s" % variable)


	def translateForInternal(self, tree,
								sbml_level=Settings.defaultSbmlLevel,
								sbml_version=Settings.defaultSbmlVersion,
								simplified=False, develop=True):

		""" Translate an SBML Tree in a Sympy Tree """

		if tree is None:
			return None

		elif tree == 0:
			return SympyInteger(0)

		elif isinstance(tree,int):
			return SympyInteger(tree)

		elif isinstance(tree, float):
			if isinf(tree):
				if tree > 0:
					return SympyInf
				else:
					return SympyMul(SympyInteger(-1), SympyInf)

			elif isnan(tree):
				return SympyNan

			return SympyFloat(tree)

		elif isinstance(tree, string_types):
			return self.translateVariableForInternal(tree, sbml_level, sbml_version, simplified, develop)

		# print libsbml.formulaToString(tree)

		if tree.isInfinity():
			return SympyInf

		elif tree.isNegInfinity():
			return SympyMul(SympyInteger(-1), SympyInf)

		elif tree.isNaN():
			return SympyNan

		elif tree.getType() == libsbml.AST_RATIONAL:
			return SympyRational(SympyInteger(tree.getNumerator()), SympyInteger(tree.getDenominator()))


		elif tree.isNumber():

			if tree.isInteger():
				return SympyInteger(tree.getInteger())

			elif tree.isReal():
				return SympyFloat(tree.getReal())

			elif tree.getType() == libsbml.AST_REAL_E:
				return SympyE

			else:
				raise SbmlException("SbmlMathReader : Unknown type of number")


		elif tree.isConstant():
			if tree.getType() == libsbml.AST_CONSTANT_E:
				return SympyE

			elif tree.getType() == libsbml.AST_CONSTANT_FALSE:
				return SympyFalse

			elif tree.getType() == libsbml.AST_CONSTANT_TRUE:
				return SympyTrue

			elif tree.getType() == libsbml.AST_CONSTANT_PI:
				return SympyPi

			elif tree.getType() == libsbml.AST_NAME_AVOGADRO or tree.getType() == libsbml.AST_TYPECODE_CSYMBOL_AVOGADRO:
				return SympySymbol("_avogadro_")

			else:
				raise SbmlException("SbmlMathReader : Unknown constant")

		elif tree.isOperator():

			if tree.getType() == libsbml.AST_PLUS:

				if tree.getNumChildren() == 0:
					return SympyInteger(0)

				else:
					t_children = []
					for i_arg in range(tree.getNumChildren()):
						t_children.append(self.ensureFloat(self.translateForInternal(tree.getChild(i_arg), sbml_level, sbml_version, simplified, develop)))
					return SympyAdd(*t_children, evaluate=False)


			elif tree.getType() == libsbml.AST_MINUS:

				if tree.getNumChildren() == 2:
					return SympyAdd(
						self.ensureFloat(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop)),
						SympyMul(
							SympyInteger(-1),
							self.ensureFloat(self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop)),
							evaluate=False
						),
						evaluate=False
					)

				elif tree.getNumChildren() == 1:
					return SympyMul(
						SympyInteger(-1),
						self.ensureFloat(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop)),
						evaluate=False
					)

				elif tree.getNumChildren() == 0:
					return SympyInteger(0)

				else:
					t_tree = SympyAdd(
						self.ensureFloat(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop)),
						SympyMul(
							SympyInteger(-1),
							self.ensureFloat(self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop)),
							evaluate=False
						),
						evaluate=False
					)
					for i_arg in range(2, tree.getNumChildren()):
						t_tree = SympyAdd(
							t_tree,
							SympyMul(
								SympyInteger(-1),
								self.ensureFloat(self.translateForInternal(tree.getChild(i_arg), sbml_level, sbml_version, simplified, develop)),
								evaluate=False
							),
							evaluate=False
						)

					return t_tree


			elif tree.getType() == libsbml.AST_TIMES:

				if tree.getNumChildren() == 0:
					return SympyInteger(1)
				else:
					t_children = []

					for i_arg in range(tree.getNumChildren()):
						t_children.append(self.ensureFloat(self.translateForInternal(tree.getChild(i_arg), sbml_level, sbml_version, simplified, develop)))

					return SympyMul(*t_children, evaluate=False)

			elif tree.getType() == libsbml.AST_DIVIDE:
				if tree.getChild(0).isInteger() and tree.getChild(1).isInteger():

					t_tree = SympyRational(
						SympyInteger(self.translateForInternal(tree.getChild(0), simplified, develop)),
						SympyInteger(self.translateForInternal(tree.getChild(1), simplified, develop)))

					return t_tree

				else:
					return SympyMul(self.ensureFloat(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop)),
								SympyPow(self.ensureFloat(self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop)), SympyInteger(-1)))

			elif tree.getType() == libsbml.AST_POWER:
				t_x = self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop)
				t_n = self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop)
				return SympyPow(t_x, t_n, evaluate=False)


		elif tree.isFunction():

				# AST_FUNCTION_ABS
			if tree.getType() == libsbml.AST_FUNCTION_ABS:
				return SympyAbs(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCCOS
			elif tree.getType() == libsbml.AST_FUNCTION_ARCCOS:
				return SympyAcos(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCCOSH
			elif tree.getType() == libsbml.AST_FUNCTION_ARCCOSH:
				return SympyAcosh(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCCOT
			elif tree.getType() == libsbml.AST_FUNCTION_ARCCOT:
				return SympyAcot(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCCOTH
			elif tree.getType() == libsbml.AST_FUNCTION_ARCCOTH:
				return SympyAcoth(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCCSC
			elif tree.getType() == libsbml.AST_FUNCTION_ARCCSC:
				return SympyAcsc(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCCSCH
			elif tree.getType() == libsbml.AST_FUNCTION_ARCCSCH:
				return SympyAsinh(SympyPow(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), SympyInteger(-1), evaluate=False), evaluate=False)

				# AST_FUNCTION_ARCSEC
			elif tree.getType() == libsbml.AST_FUNCTION_ARCSEC:
				return SympyAsec(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCSECH
			elif tree.getType() == libsbml.AST_FUNCTION_ARCSECH:
				return SympyAcosh(SympyPow(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), SympyInteger(-1), evaluate=False), evaluate=False)

				# AST_FUNCTION_ARCSIN
			elif tree.getType() == libsbml.AST_FUNCTION_ARCSIN:
				return SympyAsin(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCSINH
			elif tree.getType() == libsbml.AST_FUNCTION_ARCSINH:
				return SympyAsinh(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCTAN
			elif tree.getType() == libsbml.AST_FUNCTION_ARCTAN:
				return SympyAtan(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_ARCTANH
			elif tree.getType() == libsbml.AST_FUNCTION_ARCTANH:
				return SympyAtanh(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_CEILING
			elif tree.getType() == libsbml.AST_FUNCTION_CEILING:
				return SympyCeiling(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_COS
			elif tree.getType() == libsbml.AST_FUNCTION_COS:
				return SympyCos(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_COSH
			elif tree.getType() == libsbml.AST_FUNCTION_COSH:
				return SympyCosh(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_COT
			elif tree.getType() == libsbml.AST_FUNCTION_COT:
				return SympyCot(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_COTH
			elif tree.getType() == libsbml.AST_FUNCTION_COTH:
				return SympyCoth(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_CSC
			elif tree.getType() == libsbml.AST_FUNCTION_CSC:
				return SympyCsc(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_CSCH
			elif tree.getType() == libsbml.AST_FUNCTION_CSCH:
				# return SympyCsch(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)
				return SympyPow(SympySinh(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False),
								SympyInteger(-1), evaluate=False)
				# AST_FUNCTION_DELAY
			elif tree.getType() == libsbml.AST_FUNCTION_DELAY:
				raise TagNotImplementedModelException("CSymbolDelay")

				# AST_FUNCTION_EXP
			elif tree.getType() == libsbml.AST_FUNCTION_EXP:
				return SympyExp(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_FACTORIAL
			elif tree.getType() == libsbml.AST_FUNCTION_FACTORIAL:
				return SympyFactorial(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_FLOOR
			elif tree.getType() == libsbml.AST_FUNCTION_FLOOR:
				return SympyFloor(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_LN
			elif tree.getType() == libsbml.AST_FUNCTION_LN:
				return SympyLog(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_LOG
			elif tree.getType() == libsbml.AST_FUNCTION_LOG:
				t_n = self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop)
				t_x = self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop)

				return SympyLog(t_x, t_n, evaluate=False)

				# AST_FUNCTION_PIECEWISE
			elif tree.getType() == libsbml.AST_FUNCTION_PIECEWISE:

				i_arg = 0
				i_cond = 0
				t_pieces = []
				value_piecewise = False
				while i_arg < tree.getNumChildren():

					if (i_arg+1) < tree.getNumChildren():
						# print "here we have a full condition"
						t_value = self.ensureFloat(self.translateForInternal(tree.getChild(i_arg), sbml_level, sbml_version, simplified, develop))
						t_condition = self.ensureBool(self.translateForInternal(tree.getChild(i_arg+1), sbml_level, sbml_version, simplified, develop))

						if isinstance(t_value, bool) or (not isinstance(t_condition, bool) and (isinstance(t_condition.func, SympyUndefinedFunction) or t_condition.func == SympyPiecewise)):
							value_piecewise = True

						t_pieces.append((t_value, t_condition))
						i_arg += 2
					else:
						t_value = self.ensureFloat(self.translateForInternal(tree.getChild(i_arg), sbml_level, sbml_version, simplified, develop))
						if isinstance(t_value, bool):
							value_piecewise = True
						t_pieces.append((t_value, True))
						i_arg += 1

				if not value_piecewise:
					return SympyPiecewise(*t_pieces, evaluate=False)
				else:
					return SympyITE(t_pieces[0][1], t_pieces[0][0], t_pieces[1][0])

				# AST_FUNCTION_POWER
			elif tree.getType() == libsbml.AST_FUNCTION_POWER:
				t_x = self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop)
				t_n = self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop)
				return SympyPow(t_x, t_n, evaluate=False)

				# AST_FUNCTION_ROOT
			elif tree.getType() == libsbml.AST_FUNCTION_ROOT:
				return SympyPow(self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop),
								SympyPow(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), SympyInteger(-1), evaluate=False), evaluate=False)

				# AST_FUNCTION_SEC
			elif tree.getType() == libsbml.AST_FUNCTION_SEC:
				return SympySec(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_SECH
			elif tree.getType() == libsbml.AST_FUNCTION_SECH:
				return SympyPow(SympyCosh(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False), SympyInteger(-1), evaluate=False)

				# AST_FUNCTION_SIN
			elif tree.getType() == libsbml.AST_FUNCTION_SIN:
				return SympySin(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_SINH
			elif tree.getType() == libsbml.AST_FUNCTION_SINH:
				return SympySinh(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_TAN
			elif tree.getType() == libsbml.AST_FUNCTION_TAN:
				return SympyTan(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

				# AST_FUNCTION_TANH
			elif tree.getType() == libsbml.AST_FUNCTION_TANH:
				return SympyTanh(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

			elif tree.getType() == libsbml.AST_FUNCTION_RATE_OF:
				return SympyRateOf(self.translateForInternal(tree.getChild(0), simplified, develop))

			elif tree.getType() == libsbml.AST_FUNCTION_QUOTIENT:
				return SympyQuotient(
						self.translateForInternal(tree.getChild(0), simplified, develop),
						self.translateForInternal(tree.getChild(1), simplified, develop)
					)

			elif tree.getType() == libsbml.AST_FUNCTION_REM:
				return SympyRem(
					self.translateForInternal(tree.getChild(0), simplified, develop),
					self.translateForInternal(tree.getChild(1), simplified, develop)
				)

			elif tree.getType() == libsbml.AST_FUNCTION_MIN:
				t_args = []

				for param in range(0, tree.getNumChildren()):
					t_arg = self.translateForInternal(tree.getChild(param), sbml_level, sbml_version, simplified, develop)
					t_args.append(
						t_arg
					)

				return SympyUnevaluatedMin(*t_args)

			elif tree.getType() == libsbml.AST_FUNCTION_MAX:
				t_args = []

				for param in range(0, tree.getNumChildren()):
					t_args.append(
						self.translateForInternal(tree.getChild(param), sbml_level, sbml_version, simplified, develop)
					)

				return SympyUnevaluatedMax(*t_args)

			else:
				t_args = []

				for param in range(0, tree.getNumChildren()):
					t_args.append(self.translateForInternal(tree.getChild(param), sbml_level, sbml_version, simplified, develop))

				t_funcdef = self.model.listOfFunctionDefinitions.getBySbmlId(tree.getName())
				t_name = "_functionDefinition_%d_" % (t_funcdef.objId)
				if len(t_args) == 0:
					t_function = SympySymbol(t_name)
				else:
					t_function = SympyFunction(t_name)(*t_args)

				return t_function


		elif tree.getType() == libsbml.AST_LAMBDA:

			t_args = []
			for param in range(0, tree.getNumChildren()-1):
				t_args.append(self.translateForInternal(tree.getChild(param), sbml_level, sbml_version, simplified, develop))

			t_def = self.translateForInternal(tree.getChild(tree.getNumChildren()-1), sbml_level, sbml_version, simplified, develop)

			if len(t_args) > 0:
				return SympyLambda(tuple(t_args), t_def)
			else:
				return SympyLambda(tuple([]), t_def)

		elif tree.isRelational():

			if tree.getType() == libsbml.AST_RELATIONAL_EQ:
				t_res = SympyEqual(
						self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop),
						self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a==b && b==c && c==d, which should be equal to a==b==c==d
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyEqual(
								self.translateForInternal(tree.getChild(child-1), sbml_level, sbml_version, simplified, develop),
								self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res


			elif tree.getType() == libsbml.AST_RELATIONAL_NEQ:
				t_res = SympyUnequal(
						self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop),
						self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a!=b && b!=c && c!=d, which should be equal to a!=b!=c!=d
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyUnequal(
								self.translateForInternal(tree.getChild(child-1), sbml_level, sbml_version, simplified, develop),
								self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res


			elif tree.getType() == libsbml.AST_RELATIONAL_GT:
				t_res = SympyStrictGreaterThan(
						self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop),
						self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a>b && b>c, which should be equal to a>b>c
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyStrictGreaterThan(
								self.translateForInternal(tree.getChild(child-1), sbml_level, sbml_version, simplified, develop),
								self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res


			elif tree.getType() == libsbml.AST_RELATIONAL_LT:
				t_res = SympyStrictLessThan(
						self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop),
						self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a<b && b<c, which should be equal to a<b<c
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyStrictLessThan(
								self.translateForInternal(tree.getChild(child-1), sbml_level, sbml_version, simplified, develop),
								self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res


			elif tree.getType() == libsbml.AST_RELATIONAL_GEQ:
				t_res = SympyGreaterThan(
						self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop),
						self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a>=b && b>=c, which should be equal to a>=b>=c
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyGreaterThan(
								self.translateForInternal(tree.getChild(child-1), sbml_level, sbml_version, simplified, develop),
								self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res


			elif tree.getType() == libsbml.AST_RELATIONAL_LEQ:
				t_res = SympyLessThan(
						self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop),
						self.translateForInternal(tree.getChild(1), sbml_level, sbml_version, simplified, develop), evaluate=False)

				if tree.getNumChildren() > 2:
					# Here the idea is to make a chain, like a<=b && b<=c, which should be equal to a<=b<=c
					t_final_res = [t_res]
					for child in range(2, tree.getNumChildren()):
						t_final_res.append(SympyLessThan(
								self.translateForInternal(tree.getChild(child-1), sbml_level, sbml_version, simplified, develop),
								self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop), evaluate=False))
					return SympyAnd(*t_final_res, evaluate=False)
				else:
					return t_res


			else:
				raise SbmlException("SbmlMathReader : Unknown relational operator")
				return SympySymbol("ERR_RELATIONAL")


		elif tree.isLogical():

			if tree.getType() == libsbml.AST_LOGICAL_AND:
				t_children = []
				for child in range(0, tree.getNumChildren()):
					t_children.append(self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop))

				return SympyAnd(*t_children, evaluate=False)

			elif tree.getType() == libsbml.AST_LOGICAL_OR:
				t_children = []
				for child in range(0, tree.getNumChildren()):
					t_children.append(self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop))

				return SympyOr(*t_children, evaluate=False)

			elif tree.getType() == libsbml.AST_LOGICAL_XOR:
				t_children = []
				for child in range(0, tree.getNumChildren()):
					t_children.append(self.translateForInternal(tree.getChild(child), sbml_level, sbml_version, simplified, develop))

				return SympyXor(*t_children, evaluate=False)

			elif tree.getType() == libsbml.AST_LOGICAL_NOT:
				return SympyNot(self.translateForInternal(tree.getChild(0), sbml_level, sbml_version, simplified, develop), evaluate=False)

			elif str(tree.getName()) == "implies":
				return SympyImplies(
					self.translateForInternal(tree.getChild(0), simplified, develop),
					self.translateForInternal(tree.getChild(1), simplified, develop),
					evaluate=False
				)


			else:
				raise SbmlException("SbmlMathReader : Unknown logical operator")
				return "unknown logical operator"


		elif tree.getType() == libsbml.AST_NAME:
			return self.translateVariableForInternal(tree.getName(), sbml_level, sbml_version, simplified, develop)

		elif tree.getType() == libsbml.AST_NAME_TIME:
			return SympySymbol("_time_")

		elif tree.getType() == libsbml.AST_NAME_AVOGADRO:
			return SympySymbol("_avogadro_")

		else:
			if sbml_level <= 2:
				t_string = libsbml.formulaToString(tree)
			else:
				t_string = libsbml.formulaToL3String(tree)

			raise MathException("SbmlMathReader : Unknown mathematical term : %s" % t_string)

			return SympySymbol("ERR_UNKNOWN_TYPE")

	def ensureBool(self, tree):

		if tree is not None:

			if tree.func == SympyInteger:
				tree = SympyFalse if tree == SympyInteger(0) else SympyTrue

			elif tree.func == SympyFloat:
				tree = SympyFalse if tree == SympyFloat(0.0) else SympyTrue

			return tree

	def ensureInteger(self, tree):

		if tree is not None:
			if tree == SympyTrue:
				return SympyInteger(1)

			elif tree == SympyFalse:
				return SympyInteger(0)

			return tree

	def ensureFloat(self, tree):

		if tree is not None:
			if tree == SympyTrue:
				return SympyFloat(1)

			elif tree == SympyFalse:
				return SympyFloat(0)

			return tree


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyEqual, SympyInteger
from sympy import solve, srepr, pretty


class DAE(object):
	""" DAE class """

	def __init__ (self, model):
		""" Constructor of ode class """
		self.__model = model
		self.__definition = None

	def new(self, definition):
		self.__definition = definition

	def getDefinition(self):
		return self.__definition

	def getFormula(self, rawFormula=True, developped=False):

		if developped:
			t_definition = self.__definition.getDeveloppedInternalMathFormula(rawFormula=rawFormula)
		else:
			t_definition = self.__definition.getInternalMathFormula(rawFormula=rawFormula)

		return SympyEqual(
			t_definition,
			SympyInteger(0)
		)

	def __str__(self):
		return "%s = 0" % str(self.__definition.getDeveloppedInternalMathFormula())

	def pprint(self):
		print(
			pretty(
				SympyEqual(
					self.__definition.getDeveloppedInternalMathFormula(),
					SympyInteger(0)
				)
			)

		)

	def solve(self):

		to_solve = []
		for var in self.__definition.getDeveloppedInternalMathFormula().atoms(SympySymbol):
			variable = self.__model.listOfVariables.getBySymbol(var)

			if variable is not None and variable.isAlgebraic():
				to_solve.append(var)

		return (to_solve[0], solve(self.__definition.getDeveloppedInternalMathFormula(), to_solve))

	def isValid(self):

		return self.__definition.getInternalMathFormula() != MathFormula.ZERO



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.sedml.container.ListOfAlgorithmParameters import ListOfAlgorithmParameters
from libsignetsim.settings.Settings import Settings

class Algorithm(SedBase):

	KINSOL = "KISAO:0000282"
	CVODE = "KISAO:0000019"

	def __init__(self, document):

		SedBase.__init__(self, document)

		self.__document = document
		self.__kisaoID = None
		self.listOfAlgorithmParameters = ListOfAlgorithmParameters(document)

	def readSedml(self, algo, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.readSedml(self, algo, level, version)

		if algo.isSetKisaoID():
			self.__kisaoID = algo.getKisaoID()

		self.listOfAlgorithmParameters.readSedml(algo.getListOfAlgorithmParameters(), level, version)

	def writeSedml(self, algo, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.writeSedml(self, algo, level, version)

		if self.__kisaoID is not None:
			algo.setKisaoID(self.__kisaoID)

		self.listOfAlgorithmParameters.writeSedml(algo.getListOfAlgorithmParameters(), level, version)

	def hasRelTol(self):
		return self.listOfAlgorithmParameters.hasRelTol()

	def hasAbsTol(self):
		return self.listOfAlgorithmParameters.hasAbsTol()

	def getRelTol(self):
		return self.listOfAlgorithmParameters.getRelTol()

	def getAbsTol(self):
		return self.listOfAlgorithmParameters.getAbsTol()

	def setKinSol(self):
		self.__kisaoID = self.KINSOL

	def setCVODE(self):
		self.__kisaoID = self.CVODE


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings
from libsignetsim.model.sbml.Species import Species
from libsignetsim.model.sbml.Compartment import Compartment
from libsignetsim.model.sbml.Parameter import Parameter
from libsignetsim.model.sbml.Reaction import Reaction

class XPath(object):

	VALUE = "value"
	NAME = "name"
	ID = "id"
	METAID = "metaid"


	def __init__(self, document):
		self.__document = document
		self.__rawXPath = None

	def readSedml(self, xpath, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		self.__rawXPath = xpath

	def isModelObject(self):
		return not self.__rawXPath.startswith("#")

	def isSedmlObject(self):
		return self.__rawXPath.startswith("#")

	def getModelObject(self, sbml_model, instance=False):
		if self.isModelObject():
			return sbml_model.parentDoc.getByXPath(self.__rawXPath, instance)

	def changeModelObject(self, sbml_model, value):
		if self.isModelObject():
			sbml_model.parentDoc.setByXPath(self.__rawXPath, value, instance=True)

	def setModelObject(self, object, attribute=None):
		self.__rawXPath = object.getXPath()
		if attribute is not None:
			self.__rawXPath += "/@%s" % attribute

	def getSedmlObject(self):
		if self.isSedmlObject():
			return self.__document.listOfIds.getById(self.__rawXPath[1:])

	def setSedmlObject(self, object, attribute=None):
		self.__rawXPath = "#%s" % object.getId()

	def writeSedml(self, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		return self.__rawXPath

	def getXPath(self):
		return self.__rawXPath

	def getParentXPath(self):
		paths = self.__rawXPath.split("/")[:-1]
		return "/".join(paths)

	def getElementXPath(self):
		return self.__rawXPath.split("/")[-1]

	def getElementTag(self):
		element = self.__rawXPath.split("/")[-1]
		if ":" in element:
			return element.split(":")[-1]
		else:
			return element

	def getXPathWithoutPrefixes(self):
		paths = self.__rawXPath.split("/")
		res = []
		for path in paths:
			if ":" in path:
				res.append(path.split(":")[1])
			else:
				res.append(path)
		return "/".join(res)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Initialization of the test module for URIs

"""

from libsignetsim.uris.URI import URI
from libsignetsim.uris.GoResolver import GoResolver


from unittest import TestCase


class TestGoResolver(TestCase):

	def testGoResolver(self):

		go_uri = URI()
		go_uri.setGO('GO:0003824')
		resolver = GoResolver(go_uri.getId())
		resolver.lookup()

		self.assertEqual(resolver.getName(), "catalytic activity")
		self.assertEqual(resolver.getDefinition(), "Catalysis of a biochemical reaction at physiological temperatures. In biologically catalyzed reactions, the reactants are known as substrates, and the catalysts are naturally occurring macromolecular substances known as enzymes. Enzymes possess specific binding sites for substrates, and are usually composed wholly or largely of protein, but RNA that has catalytic activity (ribozyme) is often also regarded as enzymatic.")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is the parent class for all objects having ids/names

"""


from libsignetsim.settings.Settings import Settings
from string import punctuation
from sympy import Symbol

class HasId(object):

	def __init__(self, model):

		self.__model = model

		self.__sbmlId = None
		self.__name = None


	def new(self, name=None, sbml_id=None):

		if name is None:
			self.newName()
		else:
			self.__name = name

		if sbml_id is not None:
			self.__sbmlId = sbml_id

	def copy(self, obj, sids_subs={}):

		if obj.getSbmlId() in list(sids_subs.keys()):
			self.__sbmlId = sids_subs[obj.getSbmlId()]
		else:
			self.__sbmlId = obj.getSbmlId()
		self.setName(obj.getName())


	def newName(self):


			if self.isSpecies():
				self.__name = "Species %d" % self.objId
			elif self.isParameter():
				self.__name = "Parameter %d" % self.objId
			elif self.isCompartment():
				self.__name = "Compartment %d" % self.objId
			elif self.isReaction():
				self.__name = "Reaction %d" % self.objId

			else:
				self.__name = ""



	def readSbml(self, sbml_variable,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		if sbml_level >= 2:
			self.__sbmlId = sbml_variable.getId()
		else:
			self.__sbmlId = sbml_variable.getName()

		if sbml_variable.isSetName():
			self.__name = sbml_variable.getName()



	def writeSbml(self, sbml_variable,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		if self.__sbmlId is not None:
			if sbml_level >= 2:
				sbml_variable.setId(self.__sbmlId)
			else:
				sbml_variable.setName(self.__sbmlId)

		if self.__name is not None:
			sbml_variable.setName(self.__name)


	def getNameOrSbmlId(self):

		if self.getName() is None:
			return self.getSbmlId()
		else:
			return self.getName()


	def getSbmlId(self):
		return self.__sbmlId

	def getName(self):
		return self.__name

	def setSbmlId(self, sbml_id, prefix="", model_wide=True, list_of_var=True):

		t_sbml_id = prefix + sbml_id.strip()
		if self.__sbmlId is not None and self.__sbmlId != t_sbml_id:
			if model_wide:
				self.__model.renameSbmlId(self.__sbmlId, t_sbml_id)

			elif list_of_var:
				self.__model.listOfVariables.renameSbmlId(self.__sbmlId, t_sbml_id)

		self.__sbmlId = t_sbml_id

	def setName(self, name):
		self.__name = name

	def getIdFromName(self, name):

		res_sbml_id = name.replace(" ", "_")
		res_sbml_id = res_sbml_id.replace("-", "_")
		forbidden_chars = list(set(punctuation) - set('_'))
		for t_char in forbidden_chars:
			res_sbml_id = res_sbml_id.replace(t_char,"")
		return res_sbml_id



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for 'high level' tests, using various components

"""

from libsignetsim import Model, SteadyStatesSimulation

from unittest import TestCase


class TestSteadyStates(TestCase):
	""" Tests high level functions """

	def testNoInitialValues(self):

		m = Model()
		m.setName("Enzymatic Reaction")

		e = m.listOfSpecies.new("E")
		s = m.listOfSpecies.new("S")
		p = m.listOfSpecies.new("P")

		vmax = m.listOfParameters.new("vmax")
		km = m.listOfParameters.new("km")

		r = m.listOfReactions.new("Enzymatic reaction")
		r.listOfReactants.add(s)
		r.listOfModifiers.add(e)
		r.listOfProducts.add(p)
		r.kineticLaw.setPrettyPrintMathFormula("vmax*E*S/(km+S)")

		e.setValue(10)
		s.setValue(12)
		p.setValue(0)
		vmax.setValue(0.211)
		km.setValue(1.233)

		reference_data = 12.0

		sim = SteadyStatesSimulation(
			list_of_models=[m],
		)
		sim.run()

		results = sim.getRawData()

		self.assertAlmostEqual(reference_data, results[p.getSbmlId()][0], delta=reference_data*0.001)

	def testSeveralInitialValues(self):

		m = Model()
		m.setName("Enzymatic Reaction")

		e = m.listOfSpecies.new("E")
		s = m.listOfSpecies.new("S")
		p = m.listOfSpecies.new("P")

		vmax = m.listOfParameters.new("vmax")
		km = m.listOfParameters.new("km")

		r = m.listOfReactions.new("Enzymatic reaction")
		r.listOfReactants.add(s)
		r.listOfModifiers.add(e)
		r.listOfProducts.add(p)
		r.kineticLaw.setPrettyPrintMathFormula("vmax*E*S/(km+S)")

		e.setValue(10)
		s.setValue(12)
		p.setValue(0)
		vmax.setValue(0.211)
		km.setValue(1.233)

		input_data = [1.2, 12.0, 120.0, 1200.0]
		input_species = s

		reference_data = [1.2, 12.0, 120.0, 1200.0]

		sim = SteadyStatesSimulation(
			list_of_models=[m],
			species_input=input_species,
			list_of_initial_values=input_data,
		)
		sim.run()

		results = sim.getRawData()

		for i, t_data in enumerate(reference_data):
			self.assertAlmostEqual(t_data, results[p.getSbmlId()][i], delta=t_data*0.001)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.EventAssignedVariable import EventAssignedVariable
from libsignetsim.model.sbml.InitiallyAssignedVariable import InitiallyAssignedVariable
from libsignetsim.model.sbml.RuledVariable import RuledVariable
from libsignetsim.model.Variable import Variable
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasUnits import HasUnits
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.ModelException import InvalidXPath, SbmlException
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyInteger
from re import match


class Parameter(Variable, SbmlObject, InitiallyAssignedVariable,
						RuledVariable, EventAssignedVariable,
						HasUnits, HasParentObj):
	#
	# BACKWARD = 0
	# FORWARD = 1
	# CATALYSIS = 2
	# MICHAELIS = 3
	#

	def __init__(self, model, parent_obj, obj_id, name=None,
					local_parameter=False, reaction=None):

		SbmlObject.__init__(self, model)
		InitiallyAssignedVariable.__init__(self, model)
		RuledVariable.__init__(self, model)
		EventAssignedVariable.__init__(self, model)
		HasUnits.__init__(self, model)
		HasParentObj.__init__(self, parent_obj)

		self.__model = model
		self.objId = obj_id

		self.localParameter = local_parameter
		self.reaction = reaction

		if self.localParameter:
			Variable.__init__(self, model, Variable.PARAMETER, self.reaction)
		else:
			Variable.__init__(self, model, Variable.PARAMETER)



	def copy(self, parameter, sids_subs={}, symbols_subs={}, usids_subs={}, conversion_factor=None):

		SbmlObject.copy(self, parameter)
		HasUnits.copy(self, parameter, usids_subs=usids_subs)
		Variable.copy(self, parameter, sids_subs=sids_subs, symbols_subs=symbols_subs, conversion_factor=conversion_factor)

		if self.localParameter:
			self.symbol.setInternalMathFormula(
				SympySymbol("_local_%d_%s" % (self.reaction.objId, self.getSbmlId())))


	def new(self, name, value=1, constant=True, unit=None):

		""" Creates new compartment with default options """

		SbmlObject.new(self)
		Variable.new(self, name, Variable.PARAMETER)
		HasUnits.new(self, unit)

		self.setValue(value)
		self.constant = constant


	def getReactionId(self):

		if self.localParameter:
			return self.reaction.objId
		else:
			return None

	def readSbml(self, sbml_parameter, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads a parameter from a sbml file """

		Variable.readSbml(self, sbml_parameter, sbml_level, sbml_version)
		SbmlObject.readSbml(self, sbml_parameter, sbml_level, sbml_version)
		HasUnits.readSbml(self, sbml_parameter, sbml_level, sbml_version)


	def readSbmlVariable(self, sbml_parameter, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if self.localParameter:
			self.symbol.readSbml("_local_%d_%s" % (self.reaction.objId, sbml_parameter.getId()), sbml_level, sbml_version)
		else:
			self.symbol.readSbml(sbml_parameter.getId(), sbml_level, sbml_version)

		if sbml_parameter.isSetValue():
			self.isInitialized = True
			self.value.readSbml(sbml_parameter.getValue(), sbml_level, sbml_version)
		else:
			self.value.setInternalMathFormula(SympyInteger(1))
		if sbml_parameter.isSetConstant():
			self.constant = sbml_parameter.getConstant()
		elif sbml_level == 2:
			self.constant = True
		elif sbml_level == 1:
			self.constant = False

		if self.localParameter:
			self.constant = True

	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes a parameter to  a sbml file """

		sbml_param = sbml_model.createParameter()

		SbmlObject.writeSbml(self, sbml_param, sbml_level, sbml_version)
		Variable.writeSbml(self, sbml_param, sbml_level, sbml_version)
		HasUnits.writeSbml(self, sbml_param, sbml_level, sbml_version)


	def writeSbmlVariable(self, sbml_param, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):

		if self.isInitialized:
			sbml_param.setValue(self.getValue())

		if self.constant is not None and (sbml_level == 3 or (sbml_level == 2 and not self.constant)):
			sbml_param.setConstant(self.constant)


	def setLocalParameter(self, choice):
		""" Switches the local property of the parameter """
		self.localParameter = choice


	def isLocalParameter(self):
		""" Tests the local property of the parameter """
		return self.localParameter

	def toGlobal(self):

		symbol = str(self.symbol.getInternalMathFormula())
		pattern = "_local_%d_(.*)" % self.reaction.objId
		res_match = match(pattern, symbol)

		if res_match is None:
			raise SbmlException("Is the parameter really global ?")

		new_symbol = res_match.groups()[0]
		self.symbol.renameSbmlId(symbol, new_symbol)
		self.__model.renameSbmlId(symbol, new_symbol)

		self.reaction.listOfLocalParameters.remove(self, full_remove=False)
		self.__model.listOfParameters.append(self)
		self.localParameter = False
		self.reaction = None

	def toLocal(self, reaction):

		symbol = str(self.symbol.getInternalMathFormula())
		new_symbol = "_local_%d_%s" % (reaction.objId, symbol)
		self.symbol.renameSbmlId(symbol, new_symbol)

		self.__model.listOfParameters.remove(self, full_remove=False)
		reaction.listOfLocalParameters.add(self)

		self.__model.renameSbmlId(symbol, new_symbol)

		self.localParameter = True
		self.reaction = reaction

	def getByXPath(self, xpath):

		if len(xpath) == 0:
			return self

		if len(xpath) > 1:
			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@value":
			return self.getValue()

		elif xpath[0] == "@name":
			return self.getName()

		elif xpath[0] == "@id":
			return self.getSbmlId()


	def setByXPath(self, xpath, object):

		if len(xpath) == 0:
			return InvalidXPath("/".join(xpath))

		if len(xpath) > 1:
			return InvalidXPath("/".join(xpath))

		if xpath[0] == "@value":
			return self.setValue(object)

		elif xpath[0] == "@name":
			return self.setName(object)

		elif xpath[0] == "@id":
			return self.setSbmlId(object)


	def getXPath(self, attribute=None):

		xpath = "sbml:parameter"
		if self.__model.sbmlLevel == 1:
			xpath += "[@name='%s']" % self.getSbmlId()
		else:
			xpath += "[@id='%s']" % self.getSbmlId()

		if attribute is not None:
			xpath += "/@%s" % attribute

		return "/".join([self.getParentObj().getXPath(), xpath])


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division


from past.utils import old_div
from math import exp
from numpy.random import normal


class NoiseGenerator(object):

	def __init__(self, list_of_experimental_data={}, noise=0, sampling=None):

		self.listOfExperimentalData = list_of_experimental_data

		self.noise = noise
		self.sampling = sampling

	def applyNoiseToData(self):

		for i, experimental_data in enumerate(self.listOfExperimentalData.keys()):

			t_experimental_data = self.listOfExperimentalData[experimental_data]

			t_filtered_t = []
			t_filtered_values = []

			# Filter some time points to impose a sampling
			if self.sampling is not None:
				for j,t_time in enumerate(t_experimental_data.t):

					if not (abs((old_div(float(t_time),self.sampling) - round(old_div(float(t_time),self.sampling), 0))) > 1e-12 and float(t_time) > 0):
						t_filtered_t.append(float(t_time))
						t_filtered_values.append(float(t_experimental_data.values[j]))

			else:
				t_filtered_t = t_experimental_data.t
				t_filtered_values = t_experimental_data.values

			# Add noise to all variables
			if self.noise > 0:
				for j, value in enumerate(t_filtered_values):
					noise = exp(normal(0, self.noise))
					t_filtered_values[j] = t_filtered_values[j] * noise

			self.listOfExperimentalData[experimental_data].size = len(t_filtered_t)
			self.listOfExperimentalData[experimental_data].t = t_filtered_t
			self.listOfExperimentalData[experimental_data].values = t_filtered_values



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf

from libsignetsim.sedml.SetValue import SetValue

from libsignetsim.settings.Settings import Settings


class ListOfSetValueChanges(ListOf):

	def __init__(self, document, repeated_task):

		ListOf.__init__(self, document)

		self.__document = document
		self.__repeatedTask = repeated_task
		self.__setValueCounter = 0

	def new(self, set_value_id=None):

		t_set_value_id = set_value_id
		if t_set_value_id is None:
			t_set_value_id = "%s_setvalue_%d" % (self.__repeatedTask.getId(), self.__setValueCounter)

		set_value = SetValue(self.__document, self.__repeatedTask)
		set_value.setId(t_set_value_id)
		ListOf.append(self, set_value)
		self.__setValueCounter += 1
		return set_value

	def createSetValue(self, set_value_id=None):
		return self.new(set_value_id)

	def readSedml(self, list_of_changes, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_changes, level, version)

		for change in list_of_changes:
			t_change = SetValue(self.__document, self.__repeatedTask)
			t_change.readSedml(change, level, version)
			ListOf.append(self, t_change)
			self.__setValueCounter += 1

	def writeSedml(self, list_of_changes, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_changes, level, version)

		for change in self:
			t_change = list_of_changes.createSetValue()
			change.writeSedml(t_change, level, version)

	def getValueChanges(self):

		value_changes = {}

		for change in self:
			value_changes.update(change.getValueChange())

		return value_changes


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsbml import formulaToString
from libsignetsim.settings.Settings import Settings

class Constraint(object):
	""" Constraint definition """

	def __init__ (self, model, obj_id):

		self.model = model
		self.objId = obj_id
		self.sbmlId = None

		self.math = None
		self.message = None


	def readSbml(self, constraint, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads constraint from a sbml file """

		self.sbmlId = constraint.getId()
		self.math = formulaToString(constraint.getMath())
		self.message = constraint.getMessage()


	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes constraint to a sbml file """

		sbml_constraint = sbml_model.createConstraint()
		sbml_constraint.setId(self.sbmlId)
		sbml_constraint.setMath(self.math)
		sbml_constraint.setMessage(self.message)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.model.Model import Model
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.sbml.container.ListOfModelDefinitions import ListOfModelDefinitions
from libsignetsim.model.sbml.container.ListOfExternalModelDefinitions import ListOfExternalModelDefinitions
from libsignetsim.model.ModelInstance import ModelInstance
from libsignetsim.model.ModelException import (
	FileException, SbmlException, MissingModelException, MissingSubmodelException, InvalidXPath
)
from libsignetsim.settings.Settings import Settings

from os.path import isfile, dirname, join, basename
from libsbml import SBMLReader, SBMLDocument, writeSBMLToFile,\
					XMLFileUnreadable, XMLFileOperationError, \
					LIBSBML_CAT_UNITS_CONSISTENCY, LIBSBML_SEV_INFO, \
					LIBSBML_SEV_WARNING, \
					SBMLExtensionRegistry, readSBMLFromString
from time import time
from bioservices import BioModels


class SbmlDocument(HasParentObj):
	""" Sbml model class """


	def __init__ (self, model=None, path=None, parent_obj=None):
		""" Constructor of model class """

		HasParentObj.__init__(self, parent_obj)
		self.useCompPackage = False
		self.modelInstance = None
		self.sbmlLevel = Settings.defaultSbmlLevel
		self.sbmlVersion = Settings.defaultSbmlVersion

		self.documentPath = None
		self.documentFilename = None
		self.documentDependenciesPaths = None
		self.documentDependencies = None

		self.isCompatible = None

		if model is None:
			self.model = Model(parent_doc=self, parent_obj=self)

		else:
			self.model = model
			self.sbmlLevel = model.sbmlLevel
			self.sbmlVersion = model.sbmlVersion

		self.listOfModelDefinitions = ListOfModelDefinitions(self.model)
		self.listOfExternalModelDefinitions = ListOfExternalModelDefinitions(self.model, self)


	def getSubmodel(self, submodel_id):
		if submodel_id == self.model.getSbmlId():
			return self.model

		elif submodel_id in self.listOfModelDefinitions.sbmlIds():
			return self.listOfModelDefinitions.getBySbmlId(submodel_id)

		elif submodel_id in self.listOfExternalModelDefinitions.sbmlIds():
			return self.listOfExternalModelDefinitions.getBySbmlId(submodel_id).modelDefinition


	def enableComp(self):
		self.useCompPackage = True

	def isCompEnabled(self):
		return self.useCompPackage

	def checkCompatible(self, doc):
		""" This function checks if the documents
		needs a non-implemented package """

		for i in range(doc.getNumPlugins()):
			if doc.getPlugin(i).getPackageName() != "comp":
				return False
			else:
				return True


	def readSbml(self, sbmlDoc):

		if sbmlDoc.getNumErrors() > 0:
			if sbmlDoc.getError(0).getErrorId() == XMLFileUnreadable:
				raise FileException("Unreadable SBML file !")

			elif sbmlDoc.getError(0).getErrorId() == XMLFileOperationError:
				raise FileException("Error opening SBML file !")

			else:
				# Handle other error cases here.
				if Settings.showSbmlErrors:# and Settings.verbose:
					for error in range(0, sbmlDoc.getNumErrors()):
						print(">>> SBML Error %d : %s" % (error, sbmlDoc.getError(error).getMessage()))


		self.sbmlLevel = sbmlDoc.getLevel()
		self.sbmlVersion = sbmlDoc.getVersion()

		if self.sbmlLevel == 3 and sbmlDoc.isSetPackageRequired("comp"):
			self.useCompPackage = True

			sbmlCompPlugin = sbmlDoc.getPlugin("comp")

			try:
				self.loadExternalDocumentDependencies(sbmlDoc)
				self.listOfModelDefinitions.readSbml(
					sbmlCompPlugin.getListOfModelDefinitions(),
					self.sbmlLevel, self.sbmlVersion
				)
				self.listOfExternalModelDefinitions.readSbml(
					sbmlCompPlugin.getListOfExternalModelDefinitions(),
					self.sbmlLevel, self.sbmlVersion
				)

			except MissingModelException as e:
				raise MissingSubmodelException(e.filename)

		self.model.readSbml(sbmlDoc.getModel(), self.sbmlLevel, self.sbmlVersion)

	def readSbmlFromFile(self, sbml_filename):

		if self.documentPath is None and dirname(sbml_filename) != "":
			self.documentPath = dirname(sbml_filename)

		self.documentFilename = basename(sbml_filename)

		t_filename = sbml_filename
		if dirname(sbml_filename) == "" and self.documentPath is not None:
			t_filename = join(self.documentPath, sbml_filename)

		if not isfile(t_filename):
			raise MissingModelException(t_filename)

		if Settings.verbose >= 2:
			print("> Opening SBML file : %s" % t_filename)

		sbmlReader = SBMLReader()
		if sbmlReader == None:
			raise SbmlException("Error instanciating the SBMLReader !")

		sbmlDoc = sbmlReader.readSBML(t_filename)
		self.readSbml(sbmlDoc)

	def readSbmlFromString(self, string):

		sbmlReader = SBMLReader()
		if sbmlReader == None:
			raise SbmlException("Error instanciating the SBMLReader !")

		sbmlDoc = sbmlReader.readSBMLFromString(string)
		self.readSbml(sbmlDoc)

	def readFromBiomodels(self, biomodels_id):

		biomodels = BioModels()
		unicode_string = biomodels.getModelSBMLById(biomodels_id)
		sbml_string = unicode_string.encode('utf-8')

		if sbml_string is not None and sbml_string != "":
			self.readSbmlFromString(sbml_string)

	def writeSbml(self):

		t0 = time()

		sbmlDoc = SBMLDocument(self.sbmlLevel,self.sbmlVersion)

		# Adding xhtml namespace to write notes
		# sbmlDoc.getNamespaces().add("http://www.w3.org/1999/xhtml", "xhtml")
		sbmlDoc.setLevelAndVersion(self.sbmlLevel, self.sbmlVersion)
		sbmlDoc.setConsistencyChecks(LIBSBML_CAT_UNITS_CONSISTENCY, False)


		if self.sbmlLevel == 3 and self.useCompPackage:
			sbmlDoc.enablePackage("http://www.sbml.org/sbml/level3/version1/comp/version1", "comp", True)
			sbmlDoc.setPackageRequired("comp", True)

		sbmlModel = sbmlDoc.createModel()

		self.model.writeSbml(sbmlModel, self.sbmlLevel, self.sbmlVersion)

		if self.sbmlLevel == 3 and self.useCompPackage:
			self.saveExternalDocumentDependencies(self.documentPath)
			self.listOfModelDefinitions.writeSbml(sbmlDoc.getPlugin("comp"), self.sbmlLevel, self.sbmlVersion)
			self.listOfExternalModelDefinitions.writeSbml(sbmlDoc.getPlugin("comp"), self.sbmlLevel, self.sbmlVersion)

		if Settings.showSbmlErrors:
			sbmlDoc.validateSBML()
			for error in range(0, sbmlDoc.getNumErrors()):

				if sbmlDoc.getError(error).getSeverity() not in [LIBSBML_SEV_INFO, LIBSBML_SEV_WARNING]:
					print(">>> SBML Error %d : %s" % (error, sbmlDoc.getError(error).getMessage()))

				else:
					print(">>> SBML Warning %d : %s" % (error, sbmlDoc.getError(error).getMessage()))

		return sbmlDoc


	def writeSbmlToFile(self, sbml_filename=None, path=None):

		t0 = time()

		if sbml_filename is not None:
			self.documentPath = dirname(sbml_filename)
			self.documentFilename = basename(sbml_filename)

		if path is not None:
			self.documentPath = path

		sbmlDoc = self.writeSbml()

		# Writing the final file

		result = writeSBMLToFile(sbmlDoc, join(self.documentPath, self.documentFilename))

		if result == 1:
			return True
		else:
			raise FileException("Failed to write %s" % join(self.documentPath, self.documentFilename))

		# 	return False
		#
		# if Settings.verboseTiming >= 1:
		# 	print "Writing document %s into directory %s in %.2gs" % (self.documentFilename, self.documentPath, time()-t0)


	def getModelInstance(self, rebuild=False):
		if self.useCompPackage:
			if rebuild or self.modelInstance is None:
				t0 = time()
				self.modelInstance = ModelInstance(self.model, self)
				t1 = time()

				if Settings.verboseTiming >= 1:
					print("> Instance produced in %.2gs" % (t1-t0))

			return self.modelInstance
		else:
			return self.model

	def getLevels(self):
		return [1,2,3]


	def getVersions(self):
		if self.sbmlLevel == 1:
			return [2]
		elif self.sbmlLevel == 2:
			return [1,2,3,4,5]
		elif self.sbmlLevel == 3:
			return [1]
		else:
			return []


	def setSbmlLevel(self, level):

		self.sbmlLevel = level

		if level == 1:
			self.sbmlVersion = 2
		elif level == 2:
			self.sbmlVersion = 5
		elif level == 3:
			self.sbmlVersion = 1

	def getExternalDocumentDependencies(self, sbml_doc=None):

		if self.useCompPackage:

			self.documentDependenciesPaths = []

			if sbml_doc is None:
				for external_doc in self.listOfExternalModelDefinitions:
					self.documentDependenciesPaths.append(external_doc.getSource())
			else:
				for external_doc in sbml_doc.getPlugin("comp").getListOfExternalModelDefinitions():
					if external_doc.isSetSource():
						self.documentDependenciesPaths.append(external_doc.getSource())

			self.documentDependenciesPaths = list(set(self.documentDependenciesPaths))
			return self.documentDependenciesPaths

	def loadExternalDocumentDependencies(self, sbml_doc=None):

		if self.useCompPackage:
			if self.documentDependenciesPaths is None:
				self.getExternalDocumentDependencies(sbml_doc)

			self.documentDependencies = []

			for path in self.documentDependenciesPaths:
				t_document = SbmlDocument()
				t_document.readSbmlFromFile(join(self.documentPath, path))
				self.documentDependencies.append(t_document)


	def renameExternalDocumentDependencies(self, names):

		for external_doc in self.listOfExternalModelDefinitions:
			external_doc.setSource(names[external_doc.getSource()])


	def saveExternalDocumentDependencies(self, path=None):

		if self.useCompPackage:
			for document in self.documentDependencies:
				if path is None:
					document.writeSbmlToFile()
				else:
					document.writeSbmlToFile(path=path)

	def renameSbmlIdInSubstitutions(self, old_sbml_id, new_sbml_id, submodel_ref=None):
		for substitution in self.model.listOfSbmlObjects.getListOfSubstitutions():
			substitution.renameSbmlId(old_sbml_id, new_sbml_id, submodel_ref)

	def resolveXPath(self, selector, instance=False):

		if instance:
			if selector == "sbml:model":
				return self.getModelInstance()
			else:
				raise InvalidXPath(selector)

		else:
			if selector == "sbml:model":
				return self.model

			elif selector == "sbml:listOfModelDefinitions" and self.useCompPackage:
				return self.listOfModelDefinitions

			elif selector == "sbml:listOfExternalModelDefinitions" and self.useCompPackage:
				return self.listOfExternalModelDefinitions

			else:
				raise InvalidXPath(selector)

	def getByXPath(self, xpath, instance=False):

		if xpath.startswith("/"):
			xpath = xpath[1:]

		tokens = xpath.split("/")

		try:
			if tokens[0] == "sbml:sbml":
				return self.resolveXPath(tokens[1], instance).getByXPath(tokens[2:])
			elif tokens[0] == "sbml:model":
				return self.resolveXPath(tokens[0], instance).getByXPath(tokens[1:])
			else:
				raise InvalidXPath(xpath)

		except InvalidXPath:
			raise InvalidXPath(xpath)

	def setByXPath(self, xpath, object, instance=False):

		if xpath.startswith("/"):
			xpath = xpath[1:]

		tokens = xpath.split("/")
		try:
			if tokens[0] == "sbml:sbml":
				return self.resolveXPath(tokens[1], instance).setByXPath(tokens[2:], object)
			elif tokens[0] == "sbml:model":
				return self.resolveXPath(tokens[0], instance).setByXPath(tokens[1:], object)
			else:
				raise InvalidXPath(xpath)

		except InvalidXPath:
			raise InvalidXPath(xpath)


	def getXPath(self):
		if self.getParentObj() is not None:
			return "/".join([self.getParentObj().getXPath(), "sbml:sbml"])
		else:
			return "/sbml:sbml"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings

class OntologyTerm (object):

	def __init__(self, document):

		self.__document = document
		self.__id = None
		self.__term = None
		self.__sourceTermId = None
		self.__ontologyURI = None

	def readNuML(self, ontology_term, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		self.__id = ontology_term.getId()
		self.__term = ontology_term.getTerm()
		self.__sourceTermId = ontology_term.getSourceTermId()
		self.__ontologyURI = ontology_term.getOntologyURI()

	def writeNuML(self, ontology_term, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		ontology_term.setId(self.__id)
		ontology_term.setTerm(self.__term)
		ontology_term.setSourceTermId(self.__sourceTermId)
		ontology_term.setOntologyURI(self.__ontologyURI)

	def getTerm(self):
		return self.__term

	def getId(self):
		return self.__id

	def setId(self, ontology_term_id):
		self.__id = ontology_term_id

	def defineAsTime(self):
		self.__term = "time"
		self.__sourceTermId = "SBO:0000345"
		self.__ontologyURI = "http://www.ebi.ac.uk/sbo/"

	def defineAsConcentration(self):
		self.__term = "concentration"
		self.__sourceTermId = "SBO:0000196"
		self.__ontologyURI = "http://www.ebi.ac.uk/sbo/"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings
from libsignetsim.model.sbml.container.ListOfReplacedElements import ListOfReplacedElements
from libsignetsim.model.sbml.ReplacedBy import ReplacedBy


class HasReplacedElements(object):

	def __init__ (self, model):


		self.__model = model

		self.__hasReplacedElements = False
		self.__hasReplacedBy = False
		self.__listOfReplacedElements = None
		self.__replacedBy = None


	def readSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		""" Reads a parameter from a sbml file """

		if self.__model.parentDoc.useCompPackage:


			t_object = sbml_object.getPlugin("comp")

			if t_object.getListOfReplacedElements() is not None and len(t_object.getListOfReplacedElements()) > 0:
				self.__hasReplacedElements = True
				self.__listOfReplacedElements = ListOfReplacedElements(self.__model, self)
				self.__listOfReplacedElements.readSbml(
						t_object.getListOfReplacedElements(),
						sbml_level, sbml_version)


			if t_object.isSetReplacedBy():
				self.__hasReplacedBy = True
				self.__replacedBy = ReplacedBy(self.__model, self)
				self.__replacedBy.readSbml(
						t_object.getReplacedBy(),
						sbml_level, sbml_version)


	def writeSbml(self, sbml_object,
						sbml_level=Settings.defaultSbmlLevel,
						sbml_version=Settings.defaultSbmlVersion):

		""" Writes a parameter to  a sbml file """

		if self.__hasReplacedElements:
			self.__listOfReplacedElements.writeSbml(sbml_object.getPlugin("comp"), sbml_level, sbml_version)

		if self.__hasReplacedBy:
			self.__replacedBy.writeSbml(sbml_object.getPlugin("comp"), sbml_level, sbml_version)

	def isModified(self):
		return False


	def hasReplacedElements(self):
		return self.__hasReplacedElements

	def getReplacedElements(self):
		if self.hasReplacedElements():
			return self.__listOfReplacedElements

	def getListOfReplacedElements(self):
		if self.hasReplacedElements():
			return self.__listOfReplacedElements

	def addReplacedElement(self):
		if not self.hasReplacedElements():
			self.__hasReplacedElements = True
			self.__listOfReplacedElements = ListOfReplacedElements(self.__model, self)
		return self.__listOfReplacedElements.new()

	def removeReplacedElement(self, re_object):

		t_replaced_element = self.__listOfReplacedElements.getByReplacedElementObject(re_object)
		if t_replaced_element is not None:
			self.__listOfReplacedElements.remove(t_replaced_element)
			if len(self.__listOfReplacedElements) == 0:
				self.__hasReplacedElements = False
				self.__listOfReplacedElements = None


	def isReplaced(self):
		return self.__hasReplacedBy

	def isReplacedBy(self):
		if self.isReplaced():
			return self.__replacedBy

	def getReplacedBy(self):
		if not self.isReplaced():
			self.__hasReplacedBy = True
			self.__replacedBy = ReplacedBy(self.__model, self)

		return self.__replacedBy

	def unsetReplacedBy(self):
		self.__hasReplacedBy = False
		self.__replacedBy = None


	def copy(self, obj, prefix="", shift=0):

		if obj.hasReplacedElements():
			self.__listOfReplacedElements = ListOfReplacedElements(self.__model, self)
			self.__listOfReplacedElements.copy(obj.getListOfReplacedElements(), prefix, shift)

		if obj.isReplaced():
			self.__replacedBy = ReplacedBy(self.__model, self)
			self.__replacedBy.copy(obj.isReplacedBy(), prefix, shift)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.sympy_shortcuts import SympyEqual
from sympy import srepr, pretty

class ODE(object):
	""" ODE class """

	def __init__ (self, model):
		""" Constructor of ode class """

		self.__model = model
		self.__variable = None
		self.__definition = MathFormula(self.__model)


	def new(self, variable, definition):

		self.__variable = variable
		self.__definition = definition


	def getVariable(self):
		return self.__variable

	def getDefinition(self):
		return self.__definition

	def setVariable(self, variable):
		self.__variable = variable

	def setDefinitionMath(self, definition):
		self.__definition.setInternalMathFormula(definition)

	def getFormula(self, rawFormula=True, developped=False):

		if developped:
			t_definition = self.__definition.getDeveloppedInternalMathFormula(rawFormula=rawFormula)
		else:
			t_definition = self.__definition.getInternalMathFormula(rawFormula=rawFormula)

		if not rawFormula and self.__variable.isSpecies() and self.__variable.isConcentration():
			t_definition /= self.__variable.getCompartment().symbol.getInternalMathFormula()

		return SympyEqual(
			self.__variable.symbol.getDerivative().getInternalMathFormula(rawFormula=rawFormula),
			t_definition
		)


	def __str__(self):

		return "%s = %s" % (
			str(self.__variable.symbol.getDerivative().getDeveloppedInternalMathFormula()),
			str(self.__definition.getDeveloppedInternalMathFormula())
		)

	def pprint(self):
		print(
			pretty(
				SympyEqual(
					self.__variable.symbol.getDerivative().getDeveloppedInternalMathFormula(),
					self.__definition.getDeveloppedInternalMathFormula()
				)
			)

		)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.sedml.HasId import HasId
from libsignetsim.sedml.XPath import XPath
from libsignetsim.sedml.URI import URI
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.ModelException import InvalidXPath
from libsignetsim.sedml.SedmlException import SedmlUnknownXPATH
from libsignetsim.sedml.math.sympy_shortcuts import SympySymbol

class Variable(SedBase, HasId):

	def __init__(self, document):

		SedBase.__init__(self, document)
		HasId.__init__(self, document)

		self.__document = document
		self.__taskReference = None
		self.__modelReference = None
		self.__symbol = URI(self.__document)
		self.__target = XPath(self.__document)


	def readSedml(self, variable, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.readSedml(self, variable, level, version)
		HasId.readSedml(self, variable, level, version)

		if variable.isSetTaskReference():
			self.__taskReference = variable.getTaskReference()

		if variable.isSetModelReference():
			self.__modelReference = variable.getModelReference()

		if variable.isSetSymbol():
			self.__symbol.readSedml(variable.getSymbol())

		if variable.isSetTarget():
			self.__target.readSedml(variable.getTarget(), level, version)

	def writeSedml(self, variable, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.writeSedml(self, variable, level, version)
		HasId.writeSedml(self, variable, level, version)

		if self.__taskReference is not None:
			variable.setTaskReference(self.__taskReference)

		if self.__modelReference is not None:
			variable.setModelReference(self.__modelReference)

		if self.__symbol.getSymbol() is not None:
			variable.setSymbol(self.__symbol.writeSedml(level, version))

		if self.__target.writeSedml(level, version) is not None:
			variable.setTarget(str(self.__target.writeSedml(level, version)))

	def getTaskReference(self):
		return self.__taskReference

	def getTask(self):
		return self.__document.listOfTasks.getTask(self.__taskReference)

	def getModelReference(self):
		return self.__modelReference

	def getSymbol(self):
		return self.__symbol

	def getTarget(self):
		return self.__target

	def getSympySymbol(self):
		return SympySymbol(self.getId())

	def getData(self):

		simulation = self.__document.listOfTasks.getTask(self.__taskReference)

		if self.__symbol.getSymbol() is not None and self.__symbol.isTime():
			return {self.getSympySymbol(): simulation.getTimes()}

		elif self.__target.getXPath() is not None:
			if self.__target.isModelObject():
				try:
					sbml_model = simulation.getModel()
					sbml_object = self.__target.getModelObject(sbml_model, instance=True)
					return {self.getSympySymbol(): simulation.getResultsByVariable(sbml_object.getSbmlId())}
				except InvalidXPath:
					raise SedmlUnknownXPATH("Unknown XPath : " + self.__target.getXPath())
			# elif self.__target.isSedmlObject():
			# 	sedml_object = self.__target.getSedmlObject()
			# 	return {self.getSymbol(): []}

	def setTaskReference(self, task_reference):
		self.__taskReference = task_reference

	def setTask(self, task):
		self.__taskReference = task.getId()

	def setModelReference(self, model_reference):
		self.__modelReference = model_reference

	def setModel(self, model):
		self.__modelReference = model.getId()

	def setTarget(self, object):
		self.__target.setModelObject(object)

	def setRawTarget(self, target):
		self.__target.readSedml(target)

	def setSymbol(self, symbol):
		self.__symbol = symbol

	def setSymbolTime(self):
		self.__symbol.setTime()






#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.DataSet import DataSet
from libsignetsim.settings.Settings import Settings

class ListOfDataSets(ListOf):

	def __init__(self, document, report):

		ListOf.__init__(self, document)

		self.__document = document
		self.__report = report
		self.__dataSetCounter = 0

	def new(self, data_set_id=None):

		t_data_set_id = data_set_id
		if t_data_set_id is None:
			t_data_set_id = "%s_dataset_%d" % (self.__report.getId(), self.__dataSetCounter)

		data_set = DataSet(self.__document)
		data_set.setId(t_data_set_id)
		ListOf.append(self, data_set)
		self.__dataSetCounter += 1
		return data_set

	def createDataSet(self, data_set_id=None):
		return self.new(data_set_id)

	def readSedml(self, list_of_data_sets, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_data_sets, level, version)

		for t_data_set in list_of_data_sets:
			data_set = DataSet(self.__document)
			data_set.readSedml(t_data_set, level, version)
			ListOf.append(self, data_set)
			self.__dataSetCounter += 1

	def writeSedml(self, list_of_data_sets, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_data_sets, level, version)

		for t_data_set in self:
			data_set = list_of_data_sets.createDataSet()
			t_data_set.writeSedml(data_set, level, version)

	def getData(self):

		data = {}

		for data_set in self:
			data.update(data_set.getData())

		return data

	def getDataToGenerate(self):

		data = []
		for data_set in self:
			data.append(data_set.getDataToGenerate())

		return list(set(data))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasParentObj import HasParentObj
from libsignetsim.model.ModelException import CannotDeleteException, InvalidXPath
from libsignetsim.model.sbml.Parameter import Parameter
from libsignetsim.settings.Settings import Settings

from re import match


class ListOfParameters(ListOf, HasIds, SbmlObject, HasParentObj):
	""" Class for the ListOfParameters in a sbml model """

	def __init__ (self, model, parent_obj, are_local_parameters=False, reaction=None):

		self.__model = model

		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		SbmlObject.__init__(self, model)
		HasParentObj.__init__(self, parent_obj)

		self.are_local_parameters = are_local_parameters
		self.reaction = reaction


	def readSbml(self, sbml_list_of_parameters, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion, reaction=None):
		""" Reads parameters' list from a sbml file """

		for sbml_parameter in sbml_list_of_parameters:
			t_parameter = Parameter(self.__model, self, self.nextId(),
									local_parameter=self.are_local_parameters,
									reaction=self.reaction)
			t_parameter.readSbml(sbml_parameter, sbml_level, sbml_version)
			ListOf.add(self, t_parameter)

		SbmlObject.readSbml(self, sbml_list_of_parameters, sbml_level, sbml_version)


	def writeSbml(self, sbml_model, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes parameters' list to a sbml file """

		for parameter in self:
			parameter.writeSbml(sbml_model, sbml_level, sbml_version)

		if len(self) > 0:
			SbmlObject.writeSbml(self, sbml_model.getListOfParameters(), sbml_level, sbml_version)


	def new(self, parameter=None, value=None):
		""" Creates new parameter """

		t_parameter = Parameter(self.__model, self, self.nextId(),
								local_parameter=self.are_local_parameters,
								reaction=self.reaction)
		t_parameter.new(parameter, value=value)
		ListOf.add(self, t_parameter)
		return t_parameter


	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={}, usids_subs={}, conversion_factor=None):

		if obj not in deletions:

			SbmlObject.copy(self, obj)

			for parameter in obj:

				if parameter not in deletions:
					t_parameter = Parameter(
						self.__model, self, self.nextId(),
						local_parameter=parameter.localParameter,
						reaction=parameter.reaction
					)

					t_parameter.copy(
						parameter,
						sids_subs=sids_subs,
						symbols_subs=symbols_subs,
						usids_subs=usids_subs,
						conversion_factor=conversion_factor
					)

					ListOf.add(self, t_parameter)


	def remove(self, parameter, full_remove=True):
		""" Remove an object from the list """

		if full_remove:
			if parameter.isInRules():
				raise CannotDeleteException("Parameter in used in rules")

			if parameter.isInReactionsRates():
				raise CannotDeleteException("Parameter in used in reactions")

			self.__model.listOfVariables.removeVariable(parameter)

		ListOf.remove(self, parameter, full_remove=full_remove)


	def removeById(self, parameter_obj_id):
		""" Remove an object from the list """

		self.remove(self.getById(parameter_obj_id))


	def resolveXPath(self, selector):

		if not (selector.startswith("parameter") or selector.startswith("sbml:parameter")):
			raise InvalidXPath(selector)

		res_match = match(r'(.*)\[@(.*)=(.*)\]', selector)
		if res_match is None:
			raise InvalidXPath(selector)

		tokens = res_match.groups()
		if len(tokens) != 3:
			raise InvalidXPath(selector)

		object = None
		if tokens[1] == "id":
			object = self.getBySbmlId(tokens[2][1:-1])
		elif tokens[1] == "name":
			object = self.getByName(tokens[2][1:-1])
		elif tokens[1] == "metaid":
			object = self.getByMetaId(tokens[2][1:-1])

		if object is not None:
			return object

		# If not returned yet
		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		if len(xpath) > 0:
			return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])
		else:
			return self

	def setByXPath(self, xpath, object):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], object)

	def getXPath(self):
		return "/".join([self.getParentObj().getXPath(), "sbml:listOfParameters"])

	def addXML(self, xml_node):
		t_parameter = Parameter(self.__model, self, self.nextId(),
								local_parameter=self.are_local_parameters,
								reaction=self.reaction)
		t_parameter.readXML(xml_node)
		ListOf.add(self, t_parameter)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from os.path import join


class OptimizationParameters(object):

	def __init__(self, workingModel=None, parameters_to_fit=None, model_instance=False):
		self.__model = workingModel
		self.__modelInstance = model_instance
		self.__parameters = parameters_to_fit

	def readOptimizationOutput(self):

		f_optimized_parameters = open(join(self.getTempDirectory(), "logs/params/output"), 'r')

		result = {}
		for line in f_optimized_parameters:
			# Comments
			if line.startswith("#"):
				pass

			# Empty line
			elif not line.strip():
				pass

			else:
				data = line.strip().split(" : ")
				t_var = self.workingModel.parentDoc.getByXPath(data[0].strip(), self.__modelInstance)

				if t_var is not None:
					if self.__modelInstance:
						t_var = self.workingModel.getDefinitionVariable(t_var)[0]
					result.update({t_var: float(data[1].strip())})

		f_optimized_parameters.close()

		return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.SimpleSbmlObject import SimpleSbmlObject
from libsignetsim.model.sbml.HasReplacedElements import HasReplacedElements

from libsignetsim.settings.Settings import Settings


class SbmlObject(SimpleSbmlObject, HasReplacedElements):

	def __init__(self, model):

		self.__model = model
		SimpleSbmlObject.__init__(self, model)
		HasReplacedElements.__init__(self, model)
		self.isMarkedToBeReplaced = False
		self.isMarkedToBeReplacedBy = None
		self.isMarkedToBeDeleted = False
		self.isMarkedToBeRenamed = False
		self.hasConversionFactor = None

	def readSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		HasReplacedElements.readSbml(self, sbml_object, sbml_level, sbml_version)
		SimpleSbmlObject.readSbml(self, sbml_object, sbml_level, sbml_version)

	def writeSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):

		HasReplacedElements.writeSbml(self, sbml_object, sbml_level, sbml_version)
		SimpleSbmlObject.writeSbml(self, sbml_object, sbml_level, sbml_version)

	def copy(self, obj, prefix="", shift=0):
		SimpleSbmlObject.copy(self, obj)

	def getModel(self):
		return self.__model



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.sbml.SbmlObject import SbmlObject

from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.sympy_shortcuts import SympyInteger, SympyMul, SympyPow
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs


class EventAssignment(SbmlObject):
	""" Class definition for event assignments """

	def __init__(self, model, obj_id, event=None, math_only=False):

		self.__model = model
		self.objId = obj_id
		self.event = event
		self.__var = None
		self.__definition = MathFormula(model, MathFormula.MATH_EVENTASSIGNMENT)

		# For math submodels, where objects are not sbml objects
		self.mathOnly = math_only
		if not self.mathOnly:
			SbmlObject.__init__(self, model)

	def readSbml(self, sbml_event_assignment, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Reads event assignment from a sbml file """

		SbmlObject.readSbml(self, sbml_event_assignment, sbml_level, sbml_version)
		self.__var = self.__model.listOfVariables.getBySbmlId(sbml_event_assignment.getVariable())
		self.getVariable().addEventAssignmentBy(self.event)

		if sbml_event_assignment.getMath() is not None:
			self.__definition.readSbml(sbml_event_assignment.getMath())

			if self.getVariable().isConcentration():
				t_comp = self.getVariable().getCompartment()
				self.__definition.setInternalMathFormula(
						SympyMul(self.__definition.getInternalMathFormula(),
									t_comp.symbol.getInternalMathFormula()))

	def writeSbml(self, sbml_event, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		""" Writes event assignemnt to a sbml file """

		sbml_event_assignment = sbml_event.createEventAssignment()
		SbmlObject.writeSbml(self, sbml_event_assignment, sbml_level, sbml_version)

		sbml_event_assignment.setVariable(self.__var.getSbmlId())

		if self.__definition.getInternalMathFormula() is not None:
			t_definition = MathFormula(self.__model, MathFormula.MATH_EVENTASSIGNMENT)
			t_definition.setInternalMathFormula(self.__definition.getInternalMathFormula())

			if self.getVariable().isConcentration():
				t_comp = self.getVariable().getCompartment()
				t_definition.setInternalMathFormula(
					SympyMul(t_definition.getInternalMathFormula(),
								SympyPow(t_comp.symbol.getInternalMathFormula(),
									SympyInteger(-1))))

			sbml_event_assignment.setMath(t_definition.getSbmlMathFormula())


	def copy(self, obj, sids_subs={}, symbols_subs={}, conversion_factors={}, time_conversion=None):

		if not self.mathOnly:
			SbmlObject.copy(self, obj)

		if obj.getVariable().getSbmlId() in list(sids_subs.keys()):
			self.__var = self.__model.listOfVariables.getBySbmlId(sids_subs[obj.getVariable().getSbmlId()])
		else:
			self.__var = self.__model.listOfVariables.getBySbmlId(obj.getVariable().getSbmlId())

		self.getVariable().addEventAssignmentBy(self.event)

		if obj.getDefinition().getInternalMathFormula() is not None:

			t_convs = {}
			for var, conversion in list(conversion_factors.items()):
				t_convs.update({var: var/conversion})

			t_definition = unevaluatedSubs(obj.getDefinition().getInternalMathFormula(rawFormula=False), symbols_subs)
			t_definition = unevaluatedSubs(t_definition, t_convs)

			t_var_symbol = unevaluatedSubs(obj.getVariable().symbol.getInternalMathFormula(), symbols_subs)

			if t_var_symbol in conversion_factors:
				t_definition *= conversion_factors[t_var_symbol]

			self.__definition.setInternalMathFormula(t_definition)

	def copySubmodel(self, obj):
		self.__var = self.__model.listOfVariables.getBySymbol(obj.getVariable().symbol.getSymbol())
		if obj.getDefinition().getInternalMathFormula() is not None:
			self.__definition.setInternalMathFormula(obj.getDefinition().getDeveloppedInternalMathFormula())

	def getVariable(self):
		return self.__var

	def getVariableMath(self):
		return self.getVariable().symbol

	def setVariable(self, variable):
		if self.__var is not None:
			self.getVariable().removeEventAssignmentBy(self.event)

		self.__var = variable
		self.getVariable().addEventAssignmentBy(self.event)

	def getPrettyPrintAssignment(self):
		if self.__definition.getInternalMathFormula() is not None:
			return self.__definition.getPrettyPrintMathFormula()

	def getAssignmentMath(self):
		return self.__definition

	def setPrettyPrintAssignment(self, value, rawFormula=False):
		self.__definition.setPrettyPrintMathFormula(str(value), rawFormula)

	def getDefinition(self):
		return self.__definition

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		self.__definition.renameSbmlId(old_sbml_id, new_sbml_id)

	def isValid(self):
		return (
			self.__definition is not None
			and self.__definition.getInternalMathFormula() is not None
			and self.__definition.getDeveloppedInternalMathFormula() is not None
		)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.Plot2D import Plot2D
from libsignetsim.sedml.Report import Report
from libsignetsim.figure.SigNetSimFigure import SigNetSimFigure
from libsignetsim.settings.Settings import Settings

import libsbml
from libsedml import SEDML_OUTPUT_PLOT2D, SEDML_OUTPUT_REPORT
from six.moves import reload_module
reload_module(libsbml)

class ListOfOutputs(ListOf):

	def __init__(self, document):

		ListOf.__init__(self, document)
		self.__document = document
		self.__outputCounter = 0

	def new(self, output_type, output_id=None):

		t_output_id = output_id
		if t_output_id is None:
			t_output_id = "output_%d" % self.__outputCounter

		if output_type == SEDML_OUTPUT_PLOT2D:
			output = Plot2D(self.__document)
			output.setId(t_output_id)
			ListOf.append(self, output)
			self.__outputCounter += 1
			return output

		elif output_type == SEDML_OUTPUT_REPORT:
			output = Report(self.__document)
			output.setId(t_output_id)
			ListOf.append(self, output)
			self.__outputCounter += 1
			return output

	def createPlot2D(self, output_id=None):
		return self.new(SEDML_OUTPUT_PLOT2D, output_id)

	def createReport(self, output_id=None):
		return self.new(SEDML_OUTPUT_REPORT, output_id)

	def readSedml(self, list_of_outputs, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_outputs, level, version)

		for t_output in list_of_outputs:

			if t_output.getTypeCode() == SEDML_OUTPUT_PLOT2D:
				output = Plot2D(self.__document)
				output.readSedml(t_output, level, version)
				ListOf.append(self, output)
				self.__outputCounter += 1

			elif t_output.getTypeCode() == SEDML_OUTPUT_REPORT:
				output = Report(self.__document)
				output.readSedml(t_output, level, version)
				ListOf.append(self, output)
				self.__outputCounter += 1

	def writeSedml(self, list_of_outputs, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_outputs, level, version)

		for t_output in self:

			if isinstance(t_output, Plot2D):
				output = list_of_outputs.createPlot2D()
				t_output.writeSedml(output, level, version)

			elif isinstance(t_output, Report):
				output = list_of_outputs.createReport()
				t_output.writeSedml(output, level, version)

	def showFigures(self):

		for output in self:
			if isinstance(output, Plot2D):
				output.showFigure()

	def hasFigure(self):

		for output in self:
			if isinstance(output, Plot2D):
				return True

	def nbFigures(self):
		nb = 0
		for output in self:
			if isinstance(output, Plot2D):
				nb += 1

		return nb

	def getPlots2D(self):

		plot = []
		for output in self:
			if isinstance(output, Plot2D):
				plot.append(output)
		return plot

	def getReports(self):

		reports = []
		for output in self:
			if isinstance(output, Report):
				reports.append(output)

		return reports

	def getDataToGenerate(self):

		data = []
		for output in self:
			data += output.getDataToGenerate()

		return list(set(data))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made for 'high level' tests, using various components

"""
from __future__ import print_function

from libsignetsim.settings.Settings import Settings
from libsignetsim.model.ModelException import TagNotImplementedModelException, PackageNotImplementedModelException
from libsignetsim.model.ModelException import SbmlException
from libsignetsim.model.math.MathException import MathException
from libsignetsim.simulation.SimulationException import SimulationException
from libsignetsim.tests.biomodels.BiomodelsTestCaseSimulation import BiomodelsTestCaseSimulation

from unittest import TestCase
from time import time
from os import walk

class TestBiomodelsCompatibility(TestCase):
	""" Tests high level functions """

	CASES_PATH = "libsignetsim/tests/biomodels/cases"

	TIME_ECH = 5
	TIME_MAX = 100

	ABS_TOL = 1e-20
	REL_TOL = 1e-6

	KEEP_FILES = True
	TEST_EXPORT = False

	# Read/Write issue
	INCOMPATIBLE_CASES = []

	# COPASI Errors
	# Copasi Invalid state
	INCOMPATIBLE_CASES += ['BIOMD0000000127', 'BIOMD0000000226',
						'BIOMD0000000227', 'BIOMD0000000248',
						'BIOMD0000000250', 'BIOMD0000000255',
						'BIOMD0000000277']

	# TODO FIX
	INCOMPATIBLE_CASES += ['BIOMD0000000055', 'BIOMD0000000056',
						'BIOMD0000000059', 'BIOMD0000000071',
						'BIOMD0000000075', 'BIOMD0000000091',
						'BIOMD0000000096', 'BIOMD0000000097',
						'BIOMD0000000098', 'BIOMD0000000113',
						'BIOMD0000000118', 'BIOMD0000000121',
						'BIOMD0000000122', 'BIOMD0000000126',
						'BIOMD0000000129', 'BIOMD0000000130',
						'BIOMD0000000131', 'BIOMD0000000132',
						'BIOMD0000000133', 'BIOMD0000000134',
						'BIOMD0000000135', 'BIOMD0000000136',
						'BIOMD0000000137', 'BIOMD0000000141',
						'BIOMD0000000142', 'BIOMD0000000148',
						'BIOMD0000000158', 'BIOMD0000000161',
						'BIOMD0000000162', 'BIOMD0000000163',
						'BIOMD0000000166', 'BIOMD0000000179',
						'BIOMD0000000180', 'BIOMD0000000206',
						'BIOMD0000000208', 'BIOMD0000000234']


	START = ''
	STOP = 'BIOMD0000000200'

	def testTimeseriesBiomodels(self):

		result = True

		dirs = [dir[0].split('/')[-1] for dir in walk(self.CASES_PATH)]
		cases = [case for case in sorted(dirs) if case != "cases"]

		for i_modelId, modelId in enumerate(cases):

			if (
					(self.START != '' and i_modelId < cases.index(self.START))
				or
					(self.STOP != '' and i_modelId > cases.index(self.STOP))
			):
				pass

			elif modelId not in self.INCOMPATIBLE_CASES:

				if Settings.verbose >= 1 or Settings.verboseTiming >= 1:
					print("")

				try:
					t0 = time()
					case = BiomodelsTestCaseSimulation(

						modelId, time_ech=self.TIME_ECH, time_max=self.TIME_MAX,
						abs_tol=self.ABS_TOL, rel_tol=self.REL_TOL,
						test_export=self.TEST_EXPORT, keep_files=self.KEEP_FILES
					)
					res = case.run()

					if res >= 0:
						print("> %s : [OK] (%.2gs)" % (modelId, time()-t0))

					elif res == -2:
						print("> %s : [ERR, Incorrect values] (%.2gs)" % (modelId, time()-t0))
						result = False

					elif res == -3:
						print("> %s : [ERR, Incorrect times] (%.2gs)" % (modelId, time()-t0))
						result = False

					else:
						print("> %s : [ERR, Unknown error] (%.2gs)" % (modelId, time()-t0))
						result = False

				except TagNotImplementedModelException as e:
					print("> %s : [ERR, TAG INCOMPATIBLE] (%s)" % (modelId, e))

				except PackageNotImplementedModelException as e:
					print("> %s : [ERR, PACKAGE INCOMPATIBLE] (%s)" % (modelId, e))

				except SimulationException as e:
					print("> %s : [ERR, Simulation failed] (%s) (%.2gs)" % (modelId, e, time()-t0))
					result = False

				except MathException as e:
					print("> %s : [ERR, Math] (%s) (%.2gs)" % (modelId, e, time()-t0))
					result = False

				except SbmlException as e:
					print("> %s : [ERR, SBML] (%s) (%.2gs)" % (modelId, e, time()-t0))
					result = False

				except Exception as e:
					print("> %s : [ERR] (%s) (%.2gs)" % (modelId, e, time()-t0))
					result = False

			else:
				print("> %s : [Not Compatible]" % modelId)

		self.assertEqual(result, True)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.data.ListOfExperimentalData import ListOfExperimentalData
from libsignetsim.data.ExperimentalData import ExperimentalData
from libsignetsim.figure.SigNetSimFigure import SigNetSimFigure
from matplotlib.pyplot import show


class ExperimentalCondition(object):

	def __init__(self, name=""):

		self.listOfInitialConditions = ListOfExperimentalData()
		self.listOfExperimentalData = ListOfExperimentalData()
		self.name = name
		self.notes = ""


	def addInitialCondition(self, t=0, name="", value=0, value_dev=None, name_attribute="name", quantification_ratio=1):

		initial_condition = ExperimentalData(t, name, value, value_dev, name_attribute, quantification_ratio)
		self.listOfInitialConditions.add(initial_condition)


	def addObservation(self, t=0, name="", value=0, value_dev=0, name_attribute="name", quantification_ratio=1):

		observation = ExperimentalData(t, name, value, value_dev, name_attribute, quantification_ratio)
		self.listOfExperimentalData.add(observation)

	def read(self, list_of_initial_values, list_of_observed_values, name=""):

		self.listOfInitialConditions = list_of_initial_values
		self.listOfExperimentalData = list_of_observed_values
		self.name = name

	def readNuML(self, condition):

		self.name = condition.getIndexValue()
		self.notes = condition.getNotes()

		for data_type in condition.getContents():

			if data_type.getIndexValue() == "initial_values":
				self.listOfInitialConditions.readNuML(data_type)
			elif data_type.getIndexValue() == "observations":
				self.listOfExperimentalData.readNuML(data_type)


	def writeNuML(self, condition):

		if self.notes is not None and len(self.notes) > 0:
			condition.setNotes(self.notes)

		list_of_initial_values = condition.createCompositeValue(condition.getDescription().getContent(), "initial_values")
		self.listOfInitialConditions.writeNuML(list_of_initial_values)

		list_of_observations = condition.createCompositeValue(condition.getDescription().getContent(), "observations")
		self.listOfExperimentalData.writeNuML(list_of_observations)

	def getMaxTime(self):
		return self.listOfExperimentalData.getMaxTime()

	def getTimes(self):
		return self.listOfExperimentalData.getTimes() + self.listOfInitialConditions.getTimes()

	def getTreatedVariables(self):
		return self.listOfInitialConditions.getVariables()

	def getVariables(self):
		return self.listOfInitialConditions.getVariables() + self.listOfExperimentalData.getVariables()

	def plot(self, figure=None, plot=None, suffix="", marker="-"):

		if plot is None:

			if figure is None:
				figure = SigNetSimFigure()
			plot = figure.add_subplot(1, 1, 1)

		for i, (var, values) in enumerate(self.listOfExperimentalData.getValues().items()):
			x = [x_i for x_i, _ in values]
			y = [y_i for _, y_i in values]
			figure.plot(plot, i, x, y, y_name=var+suffix, marker=marker)

		plot.legend(loc='upper right')
		show()

		return plot



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.SedBase import SedBase
from libsignetsim.sedml.HasId import HasId
from libsignetsim.sedml.container.ListOfParameters import ListOfParameters
from libsignetsim.sedml.container.ListOfVariables import ListOfVariables
from libsignetsim.sedml.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings

class DataGenerator(SedBase, HasId):

	def __init__(self, document):

		SedBase.__init__(self, document)
		HasId.__init__(self, document)

		self.__document = document
		self.listOfVariables = ListOfVariables(self.__document, self)
		self.listOfParameters = ListOfParameters(self.__document, self)
		self.__math = MathFormula(self.__document)

		self.__data = None

	def readSedml(self, data_generator, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.readSedml(self, data_generator, level, version)
		HasId.readSedml(self, data_generator, level, version)

		self.listOfVariables.readSedml(data_generator.getListOfVariables(), level, version)
		self.listOfParameters.readSedml(data_generator.getListOfParameters(), level, version)
		self.__math.readSedml(data_generator.getMath(), level, version)

	def writeSedml(self, data_generator, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		SedBase.writeSedml(self, data_generator, level, version)
		HasId.writeSedml(self, data_generator, level, version)

		self.listOfVariables.writeSedml(data_generator.getListOfVariables(), level, version)
		self.listOfParameters.writeSedml(data_generator.getListOfParameters(), level, version)
		data_generator.setMath(self.__math.writeSedml(level, version))

	def getMath(self):
		return self.__math

	def setMath(self, math):
		self.__math = math

	def getTasksToRun(self):
		return self.listOfVariables.getTasksToRun()

	def build(self):
		# print self.listOfVariables.getData()
		self.__data = self.getMath().evaluateMathFormula(self.listOfVariables.getData(), self.listOfParameters.getSubs())

	def getData(self):
		return self.__data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ... 

"""
from __future__ import print_function
from __future__ import division


from past.utils import old_div
from libsignetsim.settings.Settings import Settings
from threading import Thread

from time import sleep, time
from os.path import isfile
from math import isnan
from re import split
from numpy import log10

class OptimizationMonitor(Thread):

	def __init__(self, parent_optimization): #, referenceModel=None, referenceData=None):

		Thread.__init__(self)
		self.daemon = True
		self.frequency = 6
		self.parentOptimization = parent_optimization


	def run(self):
		while not isfile(self.parentOptimization.getTempDirectory() + "final_score"):
			start_time = int(time())
			if isfile(self.parentOptimization.getTempDirectory() + "optim.log"):

				#Reading file
				f_optim = open(self.parentOptimization.getTempDirectory() + "optim.log","r");
				temps = []
				times = []
				for line in f_optim:
					data = split("\t", line.strip())
					temps.append(float(data[1]))
					times.append(int(data[9]))
				f_optim.close()

				#Filtering results
				#At first, the temperature will free fall and then stabilize to
				# decrease linearly.
				#We filter out the free fall part waiting for the following condition to become true :
				# derivative of the logs > -0.005
				f_temps = []
				f_times = []
				stabilized=False
				steps_ignored = old_div(Settings.defaultPlsaInitialMoves,Settings.defaultPlsaTau)-1
				for i, step in enumerate(temps):

					if i == 0:
						steps_ignored += 1

					#Here we wait for the end of the "freefall"
					elif not stabilized:
						steps_ignored += 1
						if (log10(temps[i]) - log10(temps[i-1])) > -0.001:
							stabilized = True
					else:
						#Here we also transform the standard deviation using :
						# new_temp = log10(old_temp) - log10(precision)
						#If everything is right, we can some quasi straight line until 0
						if not isnan(temps[i]):
							f_temps.append(log10(temps[i])-(log10(Settings.defaultPlsaCriterion)-1))
							f_times.append(times[i])

				#And now we compute the remaining times
				#We wait for 5 more steps, we want a smoothed average, and the first results are crap anyway
				steps_done = len(f_temps)
				if steps_done > 100:

					if isnan(f_temps[steps_done-1]) or  f_temps[steps_done-1] < 0:
						print("> Optimization %d : finishing..." % self.parentOptimization.optimizationId)
					else:
						#We compute the smoothed average
						window = steps_done - 75

						avg_der = 0
						avg_dur = 0
						for point in range(1, window):
							avg_dur += float(f_times[steps_done-point] - f_times[steps_done-point-1])
							avg_der += (f_temps[steps_done-point] - f_temps[steps_done-point-1])
						avg_dur /= float(window-1)
						avg_der /= float(window-1)

						#And the remaining steps and fime
						steps_remaining = old_div(-f_temps[steps_done-1],avg_der)
						time_remaining = steps_remaining*avg_dur

						steps_done = steps_done + steps_ignored
						ratio_done = old_div(steps_done,(steps_done + steps_remaining))

						if (not isfile(self.parentOptimization.getTempDirectory() + "final_score") and
							time_remaining > 0 and ratio_done > 0.1):
							print("> Optimization %d : %.0f minutes remaining (%d%% done, running for %.0f minutes, remaining %.0f minutes)" % (
									self.parentOptimization.optimizationId, old_div(time_remaining,60),
									int(ratio_done*100),
									old_div((times[len(times)-1]-self.parentOptimization.startTime),60),
									(((times[len(times)-1]-self.parentOptimization.startTime)*(steps_remaining))/(steps_done)/60)))


			elapsed_time = int(time()) - start_time
			sleep(max(0, (self.frequency - elapsed_time)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.MathVariable import MathVariable
from libsignetsim.model.math.sympy_shortcuts import  (
	SympySymbol, SympyInteger, SympyFloat, SympyRational, SympyAtom,
	SympyOne, SympyNegOne, SympyZero, SympyPi, SympyE, SympyExp1, SympyHalf,
	SympyInf, SympyNan, SympyAdd, SympyMul, SympyPow,
	SympyFunction, SympyUndefinedFunction, SympyLambda, SympyDerivative,
	SympyCeiling, SympyFloor, SympyAbs, SympyLog, SympyExp, SympyPiecewise,
	SympyFactorial, SympyRoot, SympyAcos, SympyAsin, SympyAtan, SympyAcosh,
	SympyAsinh, SympyAtanh, SympyCos, SympySin, SympyTan, SympyAcot,
	SympyAcoth, SympyCosh, SympySinh, SympyTanh, SympySec, SympyCsc,
	SympyCot, SympyCoth, SympyAcsc, SympyAsec,
	SympyEqual, SympyUnequal, SympyGreaterThan, SympyLessThan,
	SympyStrictGreaterThan, SympyStrictLessThan,
	SympyAnd, SympyOr, SympyXor, SympyNot, SympyTrue, SympyFalse,
	SympyMax, SympyMin)

class ListOfMathVariables(object):

	def __init__(self, model):


		self.__model = model

	def isUpToDate(self):
		return self.__upToDate


	def removeMathVariable(self, variable):

		if variable.isDerivative():
			self.__model.variablesOdes.remove(variable)
			self.__model.nbOdes -= 1

		elif variable.isAssignment():
			self.__model.variablesAssignment.remove(variable)
			self.__model.nbAssignments -= 1

		elif variable.isConstant():
			self.__model.variablesConstant.remove(variable)
			self.__model.nbConstants -= 1

		elif variable.isAlgebraic():
			self.__model.variablesAlgebraic.remove(variable)
			self.__model.nbAlgebraics -= 1


	def classifyVariables(self):

		# print ">> Starting to classify variables"
		i_variables_constant = 0
		i_variables_assignment = 0
		i_variables_odes = 0
		i_variables_algebraics = 0

		variables_odes = []
		variables_assignment = []
		variables_constant = []
		variables_algebraic = []

		for variable in self:

			if (variable.isReaction() or variable.isEvent()) or variable.isAssignmentRuled():
				# print "Assignment variable detected : %s" % variable.getSbmlId()
				variable.type = MathVariable.VAR_ASS
				variable.ind = i_variables_assignment
				variables_assignment.append(variable)
				i_variables_assignment+=1


			elif (variable.constant
					or (variable.isSpecies() and not variable.isRuled() and not variable.isInAlgebraicRules() and not variable.isInReactions(including_fast_reactions=True))
					or ((variable.isParameter() or variable.isCompartment() or variable.isStoichiometry() or variable.isConservedMoiety()) and not variable.isRuled() and not variable.isInAlgebraicRules())
					or (self.__model.sbmlLevel == 1 and variable.isCompartment() and not variable.isRuled() and not variable.isInAlgebraicRules())
					or (self.__model.sbmlLevel == 1 and variable.isParameter() and not variable.isRuled() and not variable.isInAlgebraicRules())
				):
				# print "Constant variable detected : %s (%d)" % (variable.getSbmlId(), i_variables_constant)
				variable.type = MathVariable.VAR_CST
				variable.ind = i_variables_constant
				variables_constant.append(variable)
				i_variables_constant += 1


			elif not (variable.isRuled() or (variable.isSpecies() and variable.isInReactions() or variable.hasEventAssignment())) and variable.isInAlgebraicRules():
				# print "Algebraic variable detected : %s" % variable.getSbmlId()
				variable.type = MathVariable.VAR_DAE
				variable.ind = i_variables_algebraics
				variables_algebraic.append(variable)
				i_variables_algebraics += 1

			else:
				# print "Derivative variable detected : %s" % variable.getSbmlId()
				variable.type = MathVariable.VAR_ODE
				variable.ind = i_variables_odes
				variables_odes.append(variable)
				i_variables_odes += 1


		self.__model.nbOdes = i_variables_odes
		self.__model.nbAssignments = i_variables_assignment
		self.__model.nbConstants = i_variables_constant
		self.__model.nbAlgebraics = i_variables_algebraics

		self.__model.variablesOdes = variables_odes
		self.__model.variablesAssignment = variables_assignment
		self.__model.variablesConstant = variables_constant
		self.__model.variablesAlgebraic = variables_algebraic

		self.__model.setUpToDate(True)

	def changeVariableType(self, variable, new_type):

		if variable.isDerivative():
			if variable in self.__model.variablesOdes:
				self.__model.variablesOdes.remove(variable)
				self.__model.nbOdes -= 1

		elif variable.isAssignment():
			if variable in self.__model.variablesAssignment:
				self.__model.variablesAssignment.remove(variable)
				self.__model.nbAssignments -= 1

		elif variable.isConstant():
			if variable in self.__model.variablesConstant:
				self.__model.variablesConstant.remove(variable)
				self.__model.nbConstants -= 1

		elif variable.isAlgebraic():
			if variable in self.__model.variablesAlgebraic:
				self.__model.variablesAlgebraic.remove(variable)
				self.__model.nbAlgebraics -= 1

		if new_type == MathVariable.VAR_ODE:
			self.__model.variablesOdes.append(variable)
			self.__model.nbOdes += 1

		elif new_type == MathVariable.VAR_ASS:
			self.__model.variablesAssignment.append(variable)
			self.__model.nbAssignments += 1

		elif new_type == MathVariable.VAR_CST:
			self.__model.variablesConstant.append(variable)
			self.__model.nbConstants += 1

		elif new_type == MathVariable.VAR_DAE:
			self.__model.variablesAlgebraic.append(variable)
			self.__model.nbAlgebraics += 1

		variable.type = new_type

		for i, var in enumerate(self.__model.variablesOdes):
			var.ind = i
		for i, var in enumerate(self.__model.variablesConstant):
			var.ind = i
		for i, var in enumerate(self.__model.variablesAssignment):
			var.ind = i
		for i, var in enumerate(self.__model.variablesAlgebraic):
			var.ind = i

	def copyVariables(self, model):
		# First we copy the variables list
		self.clear()

		for variable in model.listOfVariables:
			new_var = MathVariable(self.__model)
			new_var.copy(variable, pure_math_variable=True)
			self.append(new_var)


	def copySubmodel(self, model):
		# # First we copy the variables list
		self.copyVariables(model)

		self.__model.nbOdes = model.nbOdes
		self.__model.nbAssignments = model.nbAssignments
		self.__model.nbConstants = model.nbConstants
		self.__model.nbAlgebraics = model.nbAlgebraics

		self.__model.variablesOdes = []
		for var_ode in model.variablesOdes:
			t_var = self.__model.listOfVariables.getBySymbol(var_ode.symbol.getSymbol())
			self.__model.variablesOdes.append(t_var)

		self.__model.variablesAssignment = []
		for var_ass in model.variablesAssignment:
			t_var = self.__model.listOfVariables.getBySymbol(var_ass.symbol.getSymbol())
			self.__model.variablesAssignment.append(t_var)

		self.__model.variablesConstant = []
		for var_cst in model.variablesConstant:
			t_var = self.__model.listOfVariables.getBySymbol(var_cst.symbol.getSymbol())
			self.__model.variablesConstant.append(t_var)

		self.__model.variablesAlgebraic = []
		for var_alg in model.variablesAlgebraic:
			t_var = self.__model.listOfVariables.getBySymbol(var_alg.symbol.getSymbol())
			self.__model.variablesAlgebraic.append(t_var)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Initialization of the module URIs

"""


from libsignetsim.uris.NCBITaxonomyResolver import NCBITaxonomyResolver
from libsignetsim.uris.SBOResolver import SBOResolver
from libsignetsim.uris.GoResolver import GoResolver
from libsignetsim.uris.PubmedResolver import PubmedResolver
class URI(object):

	GO = 0
	TAXONOMY = 1
	BIOMODELS = 2
	PUBMED = 3
	SBO = 4
	GO_bis = 5

	IDENTIFIERS = {
		GO: 'http://identifiers.org/go/',
		GO_bis: 'http://identifiers.org/obo.go/',
		TAXONOMY: 'http://identifiers.org/taxonomy/',
		BIOMODELS: 'http://identifiers.org/biomodels.db/',
		PUBMED: 'http://identifiers.org/pubmed/',
		SBO: 'http://biomodels.net/SBO/'
	}

	def __init__(self):
		self.__rawURI = None
		self.__identifier = None
		self.__id = None

	def readURI(self, uri):

		self.__rawURI = uri
		tokens = self.__rawURI.split("/")
		self.__id = tokens[-1]
		start_token = self.__rawURI.replace(self.__id, "")

		for identifier, string in list(self.IDENTIFIERS.items()):
			if start_token.startswith(string):
				self.__identifier = identifier

		if self.__identifier is None:
			self.__id = None

	def writeURI(self):
		if self.__id is None and self.__identifier is None:
			return self.__rawURI
		else:
			return self.IDENTIFIERS[self.__identifier] + self.__id

	def __str__(self):

		if self.__rawURI is not None:
			return self.__rawURI
		else:
			return ""

	def __eq__(self, uri):

		if self.__id is not None and self.__identifier is not None:
			return self.__identifier == uri.getIdentifier() and self.__id == uri.getId()
		else:
			return self.__rawURI == uri.getRawURI()

	def getId(self):
		return self.__id
	def getIdentifier(self):
		return self.__identifier
	def getRawURI(self):
		return self.__rawURI

	def setGO(self, go_id):
		self.__identifier = self.GO
		self.__id = go_id

	def isGO(self):
		return self.__identifier == self.GO or self.__identifier == self.GO_bis


	def setBiomodels(self, biomodels_id):
		self.__identifier = self.BIOMODELS
		self.__id = biomodels_id

	def isBiomodels(self):
		return self.__identifier == self.BIOMODELS


	def setTaxonomy(self, taxonomy_id):
		self.__identifier = self.TAXONOMY
		self.__id = taxonomy_id

	def isTaxonomy(self):
		return self.__identifier == self.TAXONOMY


	def setPubmed(self, pubmed_id):
		self.__identifier = self.PUBMED
		self.__id = pubmed_id

	def isPubmed(self):
		return self.__identifier == self.PUBMED

	def setSBO(self, sbo_id):
		self.__identifier = self.SBO
		self.__id = "SBO_%.7d" % sbo_id

	def isSBO(self):
		return self.__identifier == self.SBO

	def getName(self):

		resolver = None

		if self.isTaxonomy():
			resolver = NCBITaxonomyResolver(self.__id)
		elif self.isGO():
			resolver = GoResolver(self.__id)
		elif self.isSBO():
			resolver = SBOResolver(self.__id)
		elif self.isPubmed():
			resolver = PubmedResolver(self.__id)

		try:
			resolver.lookup()
		except:
			return ""
		return resolver.getName()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.model.math.ODE import ODE

class ListOfODEs(list):
	""" Sbml model class """

	def __init__ (self, model):
		""" Constructor of model class """

		self.__model = model
		list.__init__(self)

	def build(self, including_fast_reactions=True):

		self[:] = []
		for variable in self.__model.listOfVariables:
			if variable.isDerivative():
				t_ode = ODE(self.__model)
				t_ode.new(variable,	variable.getODE(including_fast_reactions, rawFormula=True))
				list.append(self, t_ode)

	def getByVariable(self, variable):

		for ode in self:
			if ode.getVariable() == variable:
				return ode

	def __str__(self):

		res = ""
		for ode in self:
			res += str(ode) + "\n"
		return res

	def pprint(self):

		for ode in self:
			ode.pprint()
			print("\n")

